@@ -8,14 +8,19 @@
 
 #pragma once
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+
 #include <stdbool.h>
 #include <stdint.h>
 
-#include "mrcal-types.h"
+#include "types.h"
 #include "poseutils.h"
 #include "stereo.h"
 #include "triangulation.h"
-#include "mrcal-image.h"
+#include "image.h"
 
 ////////////////////////////////////////////////////////////////////////////////
 //////////////////// Lens models
@@ -108,6 +113,7 @@ mrcal_lensmodel_metadata_t mrcal_lensmodel_metadata( const mrcal_lensmodel_t* le
 // depends on the configuration. For instance, splined models use the model
 // parameters as the spline control points, so the spline density (specified in
 // the configuration) directly affects how many parameters such a model requires
+// Returns <0 on error
 int mrcal_lensmodel_num_params( const mrcal_lensmodel_t* lensmodel );
 
 
@@ -164,7 +170,6 @@ bool mrcal_project( // out
                    const mrcal_point3_t* p,
                    int N,
                    const mrcal_lensmodel_t* lensmodel,
-                   // core, distortions concatenated
                    const double* intrinsics);
 
 
@@ -191,13 +196,12 @@ bool mrcal_unproject( // out
                      const mrcal_point2_t* q,
                      int N,
                      const mrcal_lensmodel_t* lensmodel,
-                     // core, distortions concatenated
                      const double* intrinsics);
 
 
 // Project the given camera-coordinate-system points using a pinhole
 // model. See the docs for projection details:
-// http://mrcal.secretsauce.net/lensmodels.html#lensmodel-pinhole
+// https://mrcal.secretsauce.net/lensmodels.html#lensmodel-pinhole
 //
 // This is a simplified special case of mrcal_project(). We project N
 // camera-coordinate-system points p to N pixel coordinates q
@@ -216,7 +220,7 @@ void mrcal_project_pinhole( // output
 
 // Unproject the given pixel coordinates using a pinhole model.
 // See the docs for projection details:
-// http://mrcal.secretsauce.net/lensmodels.html#lensmodel-pinhole
+// https://mrcal.secretsauce.net/lensmodels.html#lensmodel-pinhole
 //
 // This is a simplified special case of mrcal_unproject(). We unproject N 2D
 // pixel coordinates q to N camera-coordinate-system vectors v. The returned
@@ -235,7 +239,7 @@ void mrcal_unproject_pinhole( // output
 
 // Project the given camera-coordinate-system points using a stereographic
 // model. See the docs for projection details:
-// http://mrcal.secretsauce.net/lensmodels.html#lensmodel-stereographic
+// https://mrcal.secretsauce.net/lensmodels.html#lensmodel-stereographic
 //
 // This is a simplified special case of mrcal_project(). We project N
 // camera-coordinate-system points p to N pixel coordinates q
@@ -254,7 +258,7 @@ void mrcal_project_stereographic( // output
 
 // Unproject the given pixel coordinates using a stereographic model.
 // See the docs for projection details:
-// http://mrcal.secretsauce.net/lensmodels.html#lensmodel-stereographic
+// https://mrcal.secretsauce.net/lensmodels.html#lensmodel-stereographic
 //
 // This is a simplified special case of mrcal_unproject(). We unproject N 2D
 // pixel coordinates q to N camera-coordinate-system vectors v. The returned
@@ -274,7 +278,7 @@ void mrcal_unproject_stereographic( // output
 
 // Project the given camera-coordinate-system points using an equirectangular
 // projection. See the docs for projection details:
-// http://mrcal.secretsauce.net/lensmodels.html#lensmodel-lonlat
+// https://mrcal.secretsauce.net/lensmodels.html#lensmodel-lonlat
 //
 // This is a simplified special case of mrcal_project(). We project N
 // camera-coordinate-system points p to N pixel coordinates q
@@ -294,7 +298,7 @@ void mrcal_project_lonlat( // output
 
 // Unproject the given pixel coordinates using an equirectangular projection.
 // See the docs for projection details:
-// http://mrcal.secretsauce.net/lensmodels.html#lensmodel-lonlat
+// https://mrcal.secretsauce.net/lensmodels.html#lensmodel-lonlat
 //
 // This is a simplified special case of mrcal_unproject(). We unproject N 2D
 // pixel coordinates q to N camera-coordinate-system vectors v. The returned
@@ -316,7 +320,7 @@ void mrcal_unproject_lonlat( // output
 
 // Project the given camera-coordinate-system points using a transverse
 // equirectangular projection. See the docs for projection details:
-// http://mrcal.secretsauce.net/lensmodels.html#lensmodel-latlon
+// https://mrcal.secretsauce.net/lensmodels.html#lensmodel-latlon
 //
 // This is a simplified special case of mrcal_project(). We project N
 // camera-coordinate-system points p to N pixel coordinates q
@@ -336,7 +340,7 @@ void mrcal_project_latlon( // output
 
 // Unproject the given pixel coordinates using a transverse equirectangular
 // projection. See the docs for projection details:
-// http://mrcal.secretsauce.net/lensmodels.html#lensmodel-latlon
+// https://mrcal.secretsauce.net/lensmodels.html#lensmodel-latlon
 //
 // This is a simplified special case of mrcal_unproject(). We unproject N 2D
 // pixel coordinates q to N camera-coordinate-system vectors v. The returned
@@ -361,8 +365,6 @@ void mrcal_unproject_latlon( // output
 //////////////////// Optimization
 ////////////////////////////////////////////////////////////////////////////////
 
-
-
 // Return the number of parameters needed in optimizing the given lens model
 //
 // This is identical to mrcal_lensmodel_num_params(), but takes into account the
@@ -466,13 +468,11 @@ mrcal_optimize( // out
 
                 // These are a seed on input, solution on output
 
-                // intrinsics is a concatenation of the intrinsics core and the
-                // distortion params. The specific distortion parameters may
-                // vary, depending on lensmodel, so this is a variable-length
-                // structure
+                // The number of intrinsics parameters varies, depending on
+                // lensmodel
                 double*             intrinsics,         // Ncameras_intrinsics * NlensParams
-                mrcal_pose_t*       extrinsics_fromref, // Ncameras_extrinsics of these. Transform FROM the reference frame
-                mrcal_pose_t*       frames_toref,       // Nframes of these.    Transform TO the reference frame
+                mrcal_pose_t*       rt_cam_ref,         // Ncameras_extrinsics of these. Transform FROM the reference frame
+                mrcal_pose_t*       rt_ref_frame,       // Nframes of these.    Transform TO the reference frame
                 mrcal_point3_t*     points,             // Npoints of these.    In the reference frame
                 mrcal_calobject_warp_t* calobject_warp,     // 1 of these. May be NULL if !problem_selections.do_optimize_calobject_warp
 
@@ -485,22 +485,29 @@ mrcal_optimize( // out
                 int Nobservations_board,
                 int Nobservations_point,
 
+                const mrcal_observation_point_triangulated_t* observations_point_triangulated,
+                int Nobservations_point_triangulated,
+
                 // All the board pixel observations, in an array of shape
                 //
                 // ( Nobservations_board,
                 //   calibration_object_height_n,
                 //   calibration_object_width_n )
                 //
-                // .x, .y are the
-                // pixel observations .z is the weight of the observation. Most
-                // of the weights are expected to be 1.0. Less precise
-                // observations have lower weights.
+                // .x, .y are the pixel observations .z is the weight of the
+                // observation. Most of the weights are expected to be 1.0. Less
+                // precise observations have lower weights.
                 //
                 // .z<0 indicates that this is an outlier. This is respected on
                 // input (even if !do_apply_outlier_rejection). New outliers are
                 // marked with .z<0 on output, so this isn't const
                 mrcal_point3_t* observations_board_pool,
 
+                // Same this, but for discrete points. Array of shape
+                //
+                // ( Nobservations_point,)
+                mrcal_point3_t* observations_point_pool,
+
                 const mrcal_lensmodel_t* lensmodel,
                 const int* imagersizes, // Ncameras_intrinsics*2 of these
                 mrcal_problem_selections_t       problem_selections,
@@ -513,10 +520,12 @@ mrcal_optimize( // out
                 bool check_gradient);
 
 
-// This is cholmod_sparse. I don't want to include the full header that defines
-// it in mrcal.h, and I don't need to: mrcal.h just needs to know that it's a
-// structure
+// These are cholmod_sparse, cholmod_factor, cholmod_common. I don't want to
+// include the full header that defines these in mrcal.h, and I don't need to:
+// mrcal.h just needs to know that these are a structure
 struct cholmod_sparse_struct;
+struct cholmod_factor_struct;
+struct cholmod_common_struct;
 
 // Evaluate the value of the callback function at the given operating point
 //
@@ -551,13 +560,11 @@ bool mrcal_optimizer_callback(// out
 
                              // in
 
-                             // intrinsics is a concatenation of the intrinsics core
-                             // and the distortion params. The specific distortion
-                             // parameters may vary, depending on lensmodel, so
-                             // this is a variable-length structure
+                             // The number of intrinsics parameters varies,
+                             // depending on lensmodel
                              const double*             intrinsics,         // Ncameras_intrinsics * NlensParams
-                             const mrcal_pose_t*       extrinsics_fromref, // Ncameras_extrinsics of these. Transform FROM the reference frame
-                             const mrcal_pose_t*       frames_toref,       // Nframes of these.    Transform TO the reference frame
+                             const mrcal_pose_t*       rt_cam_ref,         // Ncameras_extrinsics of these. Transform FROM the reference frame
+                             const mrcal_pose_t*       rt_ref_frame,       // Nframes of these.    Transform TO the reference frame
                              const mrcal_point3_t*     points,             // Npoints of these.    In the reference frame
                              const mrcal_calobject_warp_t* calobject_warp, // 1 of these. May be NULL if !problem_selections.do_optimize_calobject_warp
 
@@ -569,6 +576,9 @@ bool mrcal_optimizer_callback(// out
                              int Nobservations_board,
                              int Nobservations_point,
 
+                             const mrcal_observation_point_triangulated_t* observations_point_triangulated,
+                             int Nobservations_point_triangulated,
+
                              // All the board pixel observations, in an array of shape
                              //
                              // ( Nobservations_board,
@@ -583,6 +593,11 @@ bool mrcal_optimizer_callback(// out
                              // .z<0 indicates that this is an outlier
                              const mrcal_point3_t* observations_board_pool,
 
+                             // Same this, but for discrete points. Array of shape
+                             //
+                             // ( Nobservations_point,)
+                             const mrcal_point3_t* observations_point_pool,
+
                              const mrcal_lensmodel_t* lensmodel,
                              const int* imagersizes, // Ncameras_intrinsics*2 of these
                              mrcal_problem_selections_t       problem_selections,
@@ -592,6 +607,49 @@ bool mrcal_optimizer_callback(// out
                              int calibration_object_height_n,
                              bool verbose);
 
+bool _mrcal_drt_ref_refperturbed__dbpacked(// output
+                                          // Shape (6,Nstate_frames)
+                                          double* Kpackedf,
+                                          int Kpackedf_stride0, // in bytes. <= 0 means "contiguous"
+                                          int Kpackedf_stride1, // in bytes. <= 0 means "contiguous"
+
+                                          // Shape (6,Nstate_points)
+                                          double* Kpackedp,
+                                          int Kpackedp_stride0, // in bytes. <= 0 means "contiguous"
+                                          int Kpackedp_stride1, // in bytes. <= 0 means "contiguous"
+
+                                          // Shape (6,Nstate_calobject_warp)
+                                          double* Kpackedcw,
+                                          int Kpackedcw_stride0, // in bytes. <= 0 means "contiguous"
+                                          int Kpackedcw_stride1, // in bytes. <= 0 means "contiguous"
+
+                                          // inputs
+                                          // stuff that describes this solve
+                                          const double* b_packed,
+                                          // used only to confirm that the user passed-in the buffer they
+                                          // should have passed-in. The size must match exactly
+                                          int buffer_size_b_packed,
+
+                                          // The unitless (packed) Jacobian,
+                                          // used by the internal optimization
+                                          // routines cholmod_analyze() and
+                                          // cholmod_factorize() require
+                                          // non-const
+                                          /* const */
+                                          struct cholmod_sparse_struct* Jt,
+
+                                          // meta-parameters
+                                          int Ncameras_intrinsics, int Ncameras_extrinsics, int Nframes,
+                                          int Npoints, int Npoints_fixed, // at the end of points[]
+                                          int Nobservations_board,
+                                          int Nobservations_point,
+
+                                          const mrcal_lensmodel_t* lensmodel,
+                                          mrcal_problem_selections_t problem_selections,
+
+                                          int calibration_object_width_n,
+                                          int calibration_object_height_n);
+
 
 ////////////////////////////////////////////////////////////////////////////////
 //////////////////// Layout of the measurement and state vectors
@@ -664,7 +722,45 @@ int mrcal_measurement_index_points(int i_observation_point,
                                    int calibration_object_width_n,
                                    int calibration_object_height_n);
 int mrcal_num_measurements_points(int Nobservations_point);
-int mrcal_measurement_index_regularization(int calibration_object_width_n,
+int mrcal_measurement_index_points_triangulated(int i_point_triangulated,
+                                                int Nobservations_board,
+                                                int Nobservations_point,
+
+                                                // May be NULL if we don't have any of these
+                                                const mrcal_observation_point_triangulated_t* observations_point_triangulated,
+                                                int Nobservations_point_triangulated,
+
+                                                int calibration_object_width_n,
+                                                int calibration_object_height_n);
+int mrcal_num_measurements_points_triangulated(// May be NULL if we don't have any of these
+                                               const mrcal_observation_point_triangulated_t* observations_point_triangulated,
+                                               int Nobservations_point_triangulated);
+int mrcal_num_measurements_points_triangulated_initial_Npoints(// May be NULL if we don't have any of these
+                                                               const mrcal_observation_point_triangulated_t* observations_point_triangulated,
+                                                               int Nobservations_point_triangulated,
+
+                                                               // Only consider the leading Npoints. If Npoints < 0: take ALL the points
+                                                               int Npoints);
+bool mrcal_decode_observation_indices_points_triangulated(
+    // output
+    int* iobservation0,
+    int* iobservation1,
+    int* iobservation_point0,
+    int* Nobservations_this_point,
+    int* Nmeasurements_this_point,
+    int* ipoint,
+
+    // input
+    const int imeasurement,
+
+    const mrcal_observation_point_triangulated_t* observations_point_triangulated,
+    int Nobservations_point_triangulated);
+
+int mrcal_measurement_index_regularization(// May be NULL if we don't have any of these
+                                           const mrcal_observation_point_triangulated_t* observations_point_triangulated,
+                                           int Nobservations_point_triangulated,
+
+                                           int calibration_object_width_n,
                                            int calibration_object_height_n,
                                            int Ncameras_intrinsics, int Ncameras_extrinsics,
                                            int Nframes,
@@ -679,6 +775,11 @@ int mrcal_num_measurements_regularization(int Ncameras_intrinsics, int Ncameras_
 
 int mrcal_num_measurements(int Nobservations_board,
                            int Nobservations_point,
+
+                           // May be NULL if we don't have any of these
+                           const mrcal_observation_point_triangulated_t* observations_point_triangulated,
+                           int Nobservations_point_triangulated,
+
                            int calibration_object_width_n,
                            int calibration_object_height_n,
                            int Ncameras_intrinsics, int Ncameras_extrinsics,
@@ -733,17 +834,42 @@ int mrcal_state_index_calobject_warp(int Ncameras_intrinsics, int Ncameras_extri
 int mrcal_num_states_calobject_warp(mrcal_problem_selections_t problem_selections,
                                     int Nobservations_board);
 
-
+/////// Model-reading functions
+//
+//// These allocate memory for the model; the caller MUST
+//// mrcal_free_cameramodel(&model) when done. Return NULL on error
+//
 // if len>0, the string doesn't need to be 0-terminated. If len<=0, the end of
 // the buffer IS indicated by a '\0' byte
+mrcal_cameramodel_VOID_t* mrcal_read_cameramodel_string(const char* string,
+                                                        const int len);
+mrcal_cameramodel_VOID_t* mrcal_read_cameramodel_file  (const char* filename);
+void                      mrcal_free_cameramodel(mrcal_cameramodel_VOID_t** cameramodel);
+
+//// These read the model into a preallocated buffer *model. The given buffer is
+//// big-enough for a model with *Nintrinsics_max intrinsics. Return true on
+//// success. On failure, return false. If the error was a too-small
+//// Nintrinsics_max, a big-enough value will be reported in *Nintrinsics_max.
+//// Otherwise *Nintrinsics_max will be <= 0.
 //
-// return NULL on error
-mrcal_cameramodel_t* mrcal_read_cameramodel_string(const char* string, int len);
-mrcal_cameramodel_t* mrcal_read_cameramodel_file  (const char* filename);
-void                 mrcal_free_cameramodel(mrcal_cameramodel_t** cameramodel);
+// if len>0, the string doesn't need to be 0-terminated. If len<=0, the end of
+// the buffer IS indicated by a '\0' byte
+bool mrcal_read_cameramodel_string_into(// out
+                                   mrcal_cameramodel_VOID_t* model,
+                                   // in,out
+                                   int* Nintrinsics_max,
+                                   // in
+                                   const char* string,
+                                   const int len);
+bool mrcal_read_cameramodel_file_into  (// out
+                                   mrcal_cameramodel_VOID_t* model,
+                                   // in,out
+                                   int* Nintrinsics_max,
+                                   // in
+                                   const char* filename);
 
 bool mrcal_write_cameramodel_file(const char* filename,
-                                  const mrcal_cameramodel_t* cameramodel);
+                                  const mrcal_cameramodel_VOID_t* cameramodel);
 
 #define DECLARE_mrcal_apply_color_map(T,Tname)                          \
     bool mrcal_apply_color_map_##Tname(                                 \
@@ -785,6 +911,41 @@ DECLARE_mrcal_apply_color_map(double,   double);
 
 #undef DECLARE_mrcal_apply_color_map
 
-// Public ABI stuff, that's not for end-user consumption
-#include "mrcal-internal.h"
 
+
+// returns false on error
+typedef bool (*mrcal_callback_sensor_link_t)(const uint16_t idx_to,
+                                             const uint16_t idx_from,
+                                             void* cookie);
+
+// Traverses a connectivity graph of sensors to find the best connection from
+// the root sensor (idx==0) to every other sensor. This is useful to seed a
+// problem with sparse connections, where every sensor doesn't have overlapping
+// observations with every other sensor. See the docstring for
+// mrcal.traverse_sensor_links() for details (that Python function wraps
+// this one). Note: this C function takes a packed connectivity matrix (just the
+// upper triangle stored), while the Python function takes a full (N,N) array,
+// while assuming it is symmetric and has a 0 diagonal
+//
+// On success, the callback is called for each connection in the shortest path
+// found by this function
+//
+// returns false on error
+bool mrcal_traverse_sensor_links(const uint16_t Nsensors,
+
+                                 // (N,N) symmetric matrix with a 0 diagonal.
+                                 // I store the upper triangle only,
+                                 // row-first: a 1D array of (N*(N-1)/2)
+                                 // values. use pairwise_index() to index
+                                 const uint16_t* connectivity_matrix,
+                                 const mrcal_callback_sensor_link_t cb,
+                                 void* cookie);
+
+
+// Public ABI stuff, that's not for end-user consumption
+#include "internal.h"
+
+
+#ifdef __cplusplus
+}
+#endif
