@@ -17,8 +17,8 @@ SYNOPSIS
 
     model_joint = mrcal.cameramodel( model_for_intrinsics )
 
-    extrinsics = model_for_extrinsics.extrinsics_rt_fromref()
-    model_joint.extrinsics_rt_fromref(extrinsics)
+    rt_cam_ref = model_for_extrinsics.rt_cam_ref()
+    model_joint.rt_cam_ref(rt_cam_ref)
 
     # model_joint now has intrinsics from 'model0.cameramodel' and extrinsics
     # from 'model1.cameramodel'. I write it to disk
@@ -66,15 +66,12 @@ def _validateIntrinsics(imagersize,
     r'''Raises an exception if given components of the intrinsics is invalid'''
 
     # need two integers in the imager size
-    try:
-        N = len(imagersize)
-        if N != 2:
-            raise Exception("The imagersize must be an iterable of two positive integers")
-        if imagersize[0] <= 0 or imagersize[1] <= 0:
-            raise Exception("The imagersize must be an iterable of two positive integers")
-        if imagersize[0] != int(imagersize[0]) or imagersize[1] != int(imagersize[1]):
-            raise Exception("The imagersize must be an iterable of two positive integers")
-    except:
+    N = len(imagersize)
+    if N != 2:
+        raise Exception("The imagersize must be an iterable of two positive integers")
+    if imagersize[0] <= 0 or imagersize[1] <= 0:
+        raise Exception("The imagersize must be an iterable of two positive integers")
+    if imagersize[0] != int(imagersize[0]) or imagersize[1] != int(imagersize[1]):
         raise Exception("The imagersize must be an iterable of two positive integers")
 
     try:
@@ -137,7 +134,7 @@ def _validateValidIntrinsicsRegion(valid_intrinsics_region):
                  valid_intrinsics_region.shape[0] == 0)):
             raise Exception("The valid extrinsics region must be a numpy array of shape (N,2) with N >= 4 or N == 0")
     except:
-        raise Exception("The valid extrinsics region must be a numpy array of shape (N,2) with N >= 4. Instead got type {} of shape {}". \
+        raise Exception("The valid extrinsics region must be a numpy array of shape (N,2) with N >= 4 or N == 0. Instead got type {} of shape {}". \
                         format(type(valid_intrinsics_region), valid_intrinsics_region.shape if type(valid_intrinsics_region) is np.ndarray else None))
 
     if valid_intrinsics_region.size > 0 and \
@@ -235,12 +232,69 @@ future
 
     data_bytes = io.BytesIO()
 
+    # Skip the default values of new arguments. The defaults aren't
+    # required, and omitting them will make the models loadable with old
+    # mrcal.
+    new_keys_with_default_values_to_ignore = \
+        set(('do_apply_regularization_unity_cam01',
+             'observations_point_triangulated',
+             'indices_point_triangulated_camintrinsics_camextrinsics',))
     optimization_inputs_normalized = dict()
     for k in optimization_inputs.keys():
         v = optimization_inputs[k]
         if v is None: v = ''
+
+        if k in new_keys_with_default_values_to_ignore:
+            if isinstance(v, np.ndarray):
+                if np.size(v) == 0: continue
+            elif not v:
+                continue
+
         optimization_inputs_normalized[k] = v
 
+
+    # for legacy compatibility
+    def renamed_write_old_name(name_from, name_to, d):
+        # This is silly complex. The incoming dict might have the old name or
+        # the new name or both. The old name might be a poisoned string. I need
+        # to figure out which is the true value, I then need to put it ONLY into
+        # the old name (so that old mrcal can read these files). And a poisoned
+        # string must be unpoisoned
+
+        v_to = d.get(name_to, None)
+        if isinstance(d.get(name_from),str) and d.get(name_from).startswith('ERROR:'):
+            d[name_from] = d[name_to]
+
+        v_from = d.get(name_from, None)
+
+        if v_from is not None:
+            if v_to is None:
+                # only the old name is set; already done
+                pass
+            else:
+                # both are set. If identical, keep only the old
+                try:    equal = not np.any(v_from-v_to)
+                except: equal = False
+                if not equal:
+                    raise Exception(f"optimization_inputs has both keys '{name_from}' and '{name_to}', but they're not the same; {d[name_from]=} and {d[name_to]=}. Giving up")
+                del d[name_to]
+        else:
+            if v_to is not None:
+                d[name_from] = d[name_to]
+
+            # Delete the new name. It might have value of "None", but I don't want it at all
+            try:    del d[name_to]
+            except: pass
+
+
+    # I write the OLD name so that the old tools can read my files
+    renamed_write_old_name('frames_rt_toref',
+                           'rt_ref_frame',
+                           optimization_inputs_normalized)
+    renamed_write_old_name('extrinsics_rt_fromref',
+                           'rt_cam_ref',
+                           optimization_inputs_normalized)
+
     np.savez_compressed(data_bytes, **optimization_inputs_normalized)
     return \
         base64.b85encode(data_bytes.getvalue())
@@ -275,16 +329,42 @@ that function for details
         optimization_inputs[k] = arr
 
     # for legacy compatibility
-    def renamed(s0, s1, d):
-        if s0 in d and not s1 in d:
-            d[s1] = d[s0]
-            del d[s0]
+    def renamed(name_from, name_to, d,
+                *,
+                poisonold = False):
+        if name_from in d and not name_to in d:
+            d[name_to] = d[name_from]
+            del d[name_from]
+        if poisonold:
+            # If any python code tries to use the old names, we make sure that
+            # fails by writing a string where a matrix is expected. That code
+            # should be updated to use the new names
+            #
+            # This string must be findable by
+            # PyArray_Converter_checkrenamed_leaveNone() in mrcal-pywrap.c and
+            # renamed() in _serialize_optimization_inputs()
+            d[name_from] = f'ERROR: mrcal 2.5 renamed optimization_inputs fields: "{name_from}" -> "{name_to}". Please update your code to use the new name'
+
+
     renamed('do_optimize_intrinsic_core',
             'do_optimize_intrinsics_core',
             optimization_inputs)
     renamed('do_optimize_intrinsic_distortions',
             'do_optimize_intrinsics_distortions',
             optimization_inputs)
+    # Use the new names, poison the old names, so that the user can clearly see
+    # that they need to update their code. I can write a lot of logic to make
+    # the old code work seamlessly, but that's error-prone and I don't want to
+    # do the thorough testing required. Most likely I'm 99% of the affected
+    # population.
+    renamed('frames_rt_toref',
+            'rt_ref_frame',
+            optimization_inputs,
+            poisonold=True)
+    renamed('extrinsics_rt_fromref',
+            'rt_cam_ref',
+            optimization_inputs,
+            poisonold=True)
     # renamed('icam_intrinsics_optimization_inputs',
     #         'icam_intrinsics',
     #         optimization_inputs)
@@ -294,6 +374,9 @@ that function for details
     if 'calibration_object_height_n' in optimization_inputs:
         del optimization_inputs['calibration_object_height_n']
 
+    if 'rt_cam_ref' not in optimization_inputs or \
+       optimization_inputs['rt_cam_ref'] is None:
+        optimization_inputs['rt_cam_ref'] = np.zeros((0,6), dtype=float)
     return optimization_inputs
 
 
@@ -304,11 +387,11 @@ SYNOPSIS
 
     model = mrcal.cameramodel('xxx.cameramodel')
 
-    extrinsics_Rt_toref = model.extrinsics_Rt_toref()
+    Rt_ref_cam = model.Rt_ref_cam()
 
-    extrinsics_Rt_toref[3,2] += 10.0
+    Rt_ref_cam[3,2] += 10.0
 
-    extrinsics_Rt_toref = model.extrinsics_Rt_toref(extrinsics_Rt_toref)
+    Rt_ref_cam = model.Rt_ref_cam(Rt_ref_cam)
 
     model.write('moved.cameramodel')
 
@@ -348,10 +431,10 @@ This class represents
   represented in the reference coordinate TO a representation in the camera
   coordinate system. These are gettable/settable by these methods:
 
-  - extrinsics_rt_toref()
-  - extrinsics_rt_fromref()
-  - extrinsics_Rt_toref()
-  - extrinsics_Rt_fromref()
+  - rt_ref_cam()
+  - rt_cam_ref()
+  - Rt_ref_cam()
+  - Rt_cam_ref()
 
   These exist for convenience, and handle the necessary conversion internally.
   These make it simple to use the desired Rt/rt transformation and the desired
@@ -404,8 +487,7 @@ A sample valid .cameramodel file:
                      -0.6697132481,
                      -0.6284355415],
 
-      # extrinsics are rt_fromref
-      'extrinsics': [0,0,0,0,0,0],
+      'rt_cam_ref': [0,0,0,0,0,0],
       'imagersize': [3840,2160]
     }
 
@@ -436,9 +518,9 @@ A sample valid .cameramodel file:
                 f.write(("    [ {:.10g}, {:.10g} ],\n").format(*row))
             f.write("],\n\n")
 
-        N = len(self._extrinsics)
-        f.write("    # extrinsics are rt_fromref\n")
-        f.write(("    'extrinsics': [" + (" {:.10g}," * N) + "],\n").format(*self._extrinsics))
+        N = len(self._rt_cam_ref)
+        f.write(("    'rt_cam_ref': [" + (" {:.10g}," * N) + "],\n").format(*self._rt_cam_ref))
+        f.write(("    'extrinsics': [" + (" {:.10g}," * N) + "], # for compatibility with mrcal < 2.5\n").format(*self._rt_cam_ref))
         f.write("\n")
 
         N = 2
@@ -499,10 +581,22 @@ A sample valid .cameramodel file:
                 raise CameramodelParseException(f"Failed to parse cameramodel '{name}'")
 
         # for legacy compatibility
-        def renamed(s0, s1, d):
-            if s0 in d and not s1 in d:
-                d[s1] = d[s0]
-                del d[s0]
+        def renamed(name_from, name_to, d,
+                    *,
+                    check_equal = False):
+            if name_from in d:
+                if name_to in d:
+                    if check_equal:
+                        try:
+                            diff = np.array(d[name_from]) - np.array(d[name_to])
+                            equal = np.all( np.abs(diff) < 1e-9 )
+                            if not equal:
+                                raise CameramodelParseException(f"'{name_to}' and '{name_from}' both given, and they're NOT the same")
+                        except Exception as e:
+                            raise CameramodelParseException(f"'{name_to}' and '{name_from}' both given; couldn't check if they're the same: '{e}'")
+                else:
+                    d[name_to] = d[name_from]
+                    del d[name_from]
         renamed('distortion_model',
                 'lensmodel',
                 model)
@@ -514,11 +608,15 @@ A sample valid .cameramodel file:
                 model)
         model['lensmodel'] = model['lensmodel'].replace('DISTORTION', 'LENSMODEL')
 
+        renamed('extrinsics', 'rt_cam_ref',
+                model,
+                check_equal = True)
+
 
 
         keys_required = set(('lensmodel',
                              'intrinsics',
-                             'extrinsics',
+                             'rt_cam_ref',
                              'imagersize'))
         keys_received = set(model.keys())
         if keys_received < keys_required:
@@ -542,11 +640,11 @@ A sample valid .cameramodel file:
             warnings.warn("Invalid valid_intrinsics region; skipping: '{}'".format(e))
             valid_intrinsics_region = None
 
-        _validateExtrinsics(model['extrinsics'])
+        _validateExtrinsics(model['rt_cam_ref'])
 
         self._intrinsics                 = intrinsics
         self._valid_intrinsics_region    = mrcal.close_contour(valid_intrinsics_region)
-        self._extrinsics                 = np.array(model['extrinsics'], dtype=float)
+        self._rt_cam_ref                 = np.array(model['rt_cam_ref'], dtype=float)
         self._imagersize                 = np.array(model['imagersize'], dtype=np.int32)
 
         if 'optimization_inputs' in model:
@@ -572,6 +670,10 @@ A sample valid .cameramodel file:
                  *,
                  intrinsics              = None,
                  imagersize              = None,
+                 Rt_ref_cam              = None,
+                 Rt_cam_ref              = None,
+                 rt_ref_cam              = None,
+                 rt_cam_ref              = None,
                  extrinsics_Rt_toref     = None,
                  extrinsics_Rt_fromref   = None,
                  extrinsics_rt_toref     = None,
@@ -579,6 +681,7 @@ A sample valid .cameramodel file:
 
                  optimization_inputs     = None,
                  icam_intrinsics         = None,
+                 icam_extrinsics         = None,
 
                  valid_intrinsics_region = None ):
         r'''Initialize a new camera-model object
@@ -628,34 +731,42 @@ ARGUMENTS
 - imagersize: tuple (width,height) for the size of the imager. If given,
   'intrinsics' is also required. This may be given only as a keyword argument.
 
-- extrinsics_Rt_toref: numpy array of shape (4,3) describing the Rt
+- Rt_ref_cam: numpy array of shape (4,3) describing the Rt
   transformation FROM the camera coordinate system TO the reference coordinate
-  system. Exclusive with the other 'extrinsics_...' arguments. If given,
-  'intrinsics' and 'imagersize' are both required. If no 'extrinsics_...'
+  system. Exclusive with the other extrinsics arguments. If given,
+  'intrinsics' and 'imagersize' are both required. If no extrinsics
   arguments are given, an identity transformation is set. This may be given only
   as a keyword argument.
 
-- extrinsics_Rt_fromref: numpy array of shape (4,3) describing the Rt
+- Rt_cam_ref: numpy array of shape (4,3) describing the Rt
   transformation FROM the reference coordinate system TO the camera coordinate
-  system. Exclusive with the other 'extrinsics_...' arguments. If given,
-  'intrinsics' and 'imagersize' are both required. If no 'extrinsics_...'
+  system. Exclusive with the other extrinsics arguments. If given,
+  'intrinsics' and 'imagersize' are both required. If no extrinsics
   arguments are given, an identity transformation is set. This may be given only
   as a keyword argument.
 
-- extrinsics_rt_toref: numpy array of shape (6,) describing the rt
+- rt_ref_cam: numpy array of shape (6,) describing the rt
   transformation FROM the camera coordinate system TO the reference coordinate
-  system. Exclusive with the other 'extrinsics_...' arguments. If given,
-  'intrinsics' and 'imagersize' are both required. If no 'extrinsics_...'
+  system. Exclusive with the other extrinsics arguments. If given,
+  'intrinsics' and 'imagersize' are both required. If no extrinsics
   arguments are given, an identity transformation is set. This may be given only
   as a keyword argument.
 
-- extrinsics_rt_fromref: numpy array of shape (6,) describing the rt
+- rt_cam_ref: numpy array of shape (6,) describing the rt
   transformation FROM the reference coordinate system TO the camera coordinate
-  system. Exclusive with the other 'extrinsics_...' arguments. If given,
-  'intrinsics' and 'imagersize' are both required. If no 'extrinsics_...'
+  system. Exclusive with the other extrinsics arguments. If given,
+  'intrinsics' and 'imagersize' are both required. If no extrinsics
   arguments are given, an identity transformation is set. This may be given only
   as a keyword argument.
 
+- extrinsics_Rt_toref: legacy alias for Rt_ref_cam
+
+- extrinsics_Rt_fromref: legacy alias for Rt_cam_ref
+
+- extrinsics_rt_toref: legacy alias for rt_ref_cam
+
+- extrinsics_rt_fromref: legacy alias for rt_cam_ref
+
 - optimization_inputs: a dict of arguments to mrcal.optimize() at the optimum.
   These contain all the information needed to populate the camera model (and
   more!). If given, 'icam_intrinsics' is also required. This may be given only
@@ -665,6 +776,15 @@ ARGUMENTS
   'optimization_inputs'. If given, 'optimization_inputs' is required. This may
   be given only as a keyword argument.
 
+- icam_extrinsics: optional integer identifying this camera in the solve defined
+  by 'optimization_inputs'. If given, 'optimization_inputs' is required. This
+  may be given only as a keyword argument. If icam_extrinsics<0 we set the
+  extrinsics to the identify transformation. If icam_extrinsics>=0 we look up
+  the extrinsics in the optimization_inputs. If omitted, we call
+  corresponding_icam_extrinsics() to find the unique pose of this camera; this
+  works ONLY if we have a stationary-camera scenario. If we DO have such a
+  scenario, omitting icam_extrinsics is recommended
+
 - valid_intrinsics_region': numpy array of shape (N,2). Defines a closed contour
   in the imager pixel space. Points inside this contour are assumed to have
   'valid' intrinsics, with the meaning of 'valid' defined by the user. An array
@@ -682,6 +802,18 @@ ARGUMENTS
         if intrinsics    is not None: Nargs['discrete']      += 1
         if imagersize    is not None: Nargs['discrete']      += 1
 
+        if Rt_ref_cam is not None:
+            Nargs['discrete']   += 1
+            Nargs['extrinsics'] += 1
+        if Rt_cam_ref is not None:
+            Nargs['discrete']   += 1
+            Nargs['extrinsics'] += 1
+        if rt_ref_cam is not None:
+            Nargs['discrete']   += 1
+            Nargs['extrinsics'] += 1
+        if rt_cam_ref is not None:
+            Nargs['discrete']   += 1
+            Nargs['extrinsics'] += 1
         if extrinsics_Rt_toref is not None:
             Nargs['discrete']   += 1
             Nargs['extrinsics'] += 1
@@ -699,7 +831,8 @@ ARGUMENTS
             Nargs['optimization_inputs'] += 1
         if icam_intrinsics is not None:
             Nargs['optimization_inputs'] += 1
-
+        if icam_extrinsics is not None:
+            Nargs['optimization_inputs'] += 1
 
 
         if Nargs['file_or_model']:
@@ -709,7 +842,7 @@ ARGUMENTS
 
             if isinstance(file_or_model, cameramodel):
                 self._imagersize = np.array(file_or_model._imagersize, dtype=np.int32)
-                self._extrinsics = np.array(file_or_model._extrinsics, dtype=float)
+                self._rt_cam_ref = np.array(file_or_model._rt_cam_ref, dtype=float)
                 self._intrinsics = (str(file_or_model._intrinsics[0]),
                                     np.array(file_or_model._intrinsics[1], dtype=float))
                 if file_or_model._valid_intrinsics_region is not None:
@@ -802,14 +935,20 @@ So for the purposes of extrinsics, the reference is the "left-rectified" camera
                 def load():
                     try:
                         import yaml
-                        return yaml.safe_load(modelstring)
+                        loaded = yaml.safe_load(modelstring)
+                        if not isinstance(loaded, dict):
+                            raise CameramodelParseException("yaml parsing did not generate a dictionary")
+                        return loaded
                     except Exception as e:
                         e1 = e
                         pass
 
                     try:
                         import json
-                        return json.loads(modelstring)
+                        loaded = json.loads(modelstring)
+                        if not isinstance(loaded, dict):
+                            raise CameramodelParseException("json parsing did not generate a dictionary")
+                        return loaded
                     except Exception as e:
                         e2 = e
                         pass
@@ -959,8 +1098,8 @@ general function is used to find the data. The args are:
 
                 try:
                     model['intrinsics'] = \
-                          [M[0,0],M[1,1],M[0,2],M[1,2]] + \
-                          list(distortion)
+                          [float(x) for x in [M[0,0],M[1,1],M[0,2],M[1,2]]] + \
+                          [float(x) for x in distortion]
                 except Exception as e:
                     raise CameramodelParseException(f"No model {distortion_at}")
                 # not checking len(distortion_coefficients);
@@ -991,8 +1130,8 @@ general function is used to find the data. The args are:
                 # sit at -P[:,3] / P[0,0]
                 Rt_ref_cam[ 3,:] = -P[:,3] / P[0,0]
 
-                # extrinsics are rt_fromref
-                model['extrinsics'] = list(mrcal.rt_from_Rt(mrcal.invert_Rt(Rt_ref_cam)))
+                model['rt_cam_ref'] = \
+                    [float(x) for x in mrcal.rt_from_Rt(mrcal.invert_Rt(Rt_ref_cam))]
 
                 return repr(model)
 
@@ -1070,18 +1209,22 @@ general function is used to find the data. The args are:
                 raise Exception("discrete values specified, so none of the other inputs should be")
 
             if Nargs['discrete']-Nargs['extrinsics'] != 2:
-                raise Exception("Discrete values given. Must have gotten 'intrinsics' AND 'imagersize' AND optionally ONE of the extrinsics_...")
+                raise Exception("Discrete values given. Must have gotten 'intrinsics' AND 'imagersize' AND optionally ONE of the extrinsics")
 
             if Nargs['extrinsics'] == 0:
                 # No extrinsics. Use the identity
-                self.extrinsics_rt_fromref(np.zeros((6,),dtype=float))
+                self.rt_cam_ref(np.zeros((6,),dtype=float))
             elif Nargs['extrinsics'] == 1:
-                if   extrinsics_Rt_toref   is not None: self.extrinsics_Rt_toref  (extrinsics_Rt_toref)
+                if   Rt_ref_cam            is not None: self.Rt_ref_cam           (Rt_ref_cam)
+                elif Rt_cam_ref            is not None: self.Rt_cam_ref           (Rt_cam_ref)
+                elif rt_ref_cam            is not None: self.rt_ref_cam           (rt_ref_cam)
+                elif rt_cam_ref            is not None: self.rt_cam_ref           (rt_cam_ref)
+                elif extrinsics_Rt_toref   is not None: self.extrinsics_Rt_toref  (extrinsics_Rt_toref)
                 elif extrinsics_Rt_fromref is not None: self.extrinsics_Rt_fromref(extrinsics_Rt_fromref)
                 elif extrinsics_rt_toref   is not None: self.extrinsics_rt_toref  (extrinsics_rt_toref)
                 elif extrinsics_rt_fromref is not None: self.extrinsics_rt_fromref(extrinsics_rt_fromref)
             else:
-                raise Exception("At most one of the extrinsics_... arguments may be given")
+                raise Exception("At most one of the extrinsics arguments may be given")
 
             self.intrinsics(intrinsics, imagersize=imagersize)
 
@@ -1090,7 +1233,10 @@ general function is used to find the data. The args are:
                Nargs['discrete']:
                 raise Exception("optimization_inputs specified, so none of the other inputs should be")
 
-            if Nargs['optimization_inputs'] != 2:
+            # Not looking at Nargs['optimization_inputs'] here because
+            # icam_extrinsics is optional. so ==2 may or may not be valid
+            if optimization_inputs is None or \
+               icam_intrinsics is None:
                 raise Exception("optimization_input given. Must have gotten 'optimization_input' AND 'icam_intrinsics'")
 
             self.intrinsics( ( optimization_inputs['lensmodel'],
@@ -1099,12 +1245,13 @@ general function is used to find the data. The args are:
                              optimization_inputs = optimization_inputs,
                              icam_intrinsics     = icam_intrinsics)
 
-            icam_extrinsics = mrcal.corresponding_icam_extrinsics(icam_intrinsics,
-                                                                  **optimization_inputs)
+            if icam_extrinsics is None:
+                icam_extrinsics = mrcal.corresponding_icam_extrinsics(icam_intrinsics,
+                                                                      **optimization_inputs)
             if icam_extrinsics < 0:
-                self.extrinsics_rt_fromref(np.zeros((6,), dtype=float))
+                self.rt_cam_ref(mrcal.identity_rt())
             else:
-                self.extrinsics_rt_fromref(optimization_inputs['extrinsics_rt_fromref'][icam_extrinsics])
+                self.rt_cam_ref(optimization_inputs['rt_cam_ref'][icam_extrinsics])
 
         else:
             raise Exception("At least one source of initialization data must have been given. Need a filename or a cameramodel object or discrete arrays or optimization_inputs")
@@ -1134,7 +1281,7 @@ general function is used to find the data. The args are:
 
         funcs = (self.imagersize,
                  self.intrinsics,
-                 self.extrinsics_rt_fromref,
+                 self.rt_cam_ref,
                  self.valid_intrinsics_region)
 
         return 'mrcal.cameramodel(' + \
@@ -1191,7 +1338,7 @@ None
         _validateIntrinsics(self._imagersize,
                             self._intrinsics)
         _validateValidIntrinsicsRegion(self._valid_intrinsics_region)
-        _validateExtrinsics(self._extrinsics)
+        _validateExtrinsics(self._rt_cam_ref)
 
 
 
@@ -1209,7 +1356,7 @@ rectification matrices
 
 
 This is documented here: https://wiki.ros.org/camera_calibration_parsers
-
+                         https://docs.ros.org/en/noetic/api/sensor_msgs/html/msg/CameraInfo.html
 A sample file:
 
   image_width: 2448
@@ -1242,7 +1389,7 @@ A sample file:
                 distortion_model = 'plumb_bob'
                 distortions      = nps.glue(self._intrinsics[1][4:], 0,
                                             axis = -1)
-            if   self._intrinsics[0] == 'LENSMODEL_PINHOLE':
+            elif   self._intrinsics[0] == 'LENSMODEL_PINHOLE':
                 distortion_model = 'plumb_bob'
                 distortions      = nps.glue(self._intrinsics[1][4:],
                                             np.zeros((5,), dtype=float),
@@ -1418,59 +1565,21 @@ tuple where
             self._optimization_inputs_string = None
             self._icam_intrinsics            = None
 
-
-    def _extrinsics_rt(self, toref, rt=None):
-        r'''Get or set the extrinsics in this model
-
-This function represents the pose as a 6-long numpy array that contains
-a 3-long Rodrigues rotation followed by a 3-long translation in the last
-row:
-
-  r = rt[:3]
-  t = rt[3:]
-  R = mrcal.R_from_r(r)
-
-The transformation is b <-- R*a + t:
-
-  import numpysane as nps
-  b = nps.matmult(a, nps.transpose(R)) + t
-
-if rt is None: this is a getter; otherwise a setter.
-
-toref is a boolean. if toref: then rt maps points in the coord system of
-THIS camera to the reference coord system. Otherwise in the opposite
-direction
-
-        '''
-
-        # The internal representation is rt_fromref
-
-        if rt is None:
-            # getter
-            if not toref:
-                return np.array(self._extrinsics)
-            return mrcal.invert_rt(self._extrinsics)
+        # New intrinsics. Previous valid-intrinsics region is no longer
+        # meaningful
+        self.valid_intrinsics_region_reset()
 
 
-        # setter
-        if not toref:
-            self._extrinsics = np.array(rt, dtype=float)
-            return True
-
-        self._extrinsics = mrcal.invert_rt(rt.astype(float))
-        return True
-
-
-    def extrinsics_rt_toref(self, rt=None):
+    def rt_ref_cam(self, rt=None):
         r'''Get or set the extrinsics in this model
 
 SYNOPSIS
 
     # getter
-    rt_rc = model.extrinsics_rt_toref()
+    rt_rc = model.rt_ref_cam()
 
     # setter
-    model.extrinsics_rt_toref( rt_rc )
+    model.rt_ref_cam( rt_rc )
 
 This function gets/sets rt_toref: a numpy array of shape (6,) describing the rt
 transformation FROM the camera coordinate system TO the reference coordinate
@@ -1492,19 +1601,26 @@ If this is a getter (no arguments given), returns a a numpy array of shape (6,).
 The rt transformation TO the reference coordinate system.
 
         '''
-        return self._extrinsics_rt(True, rt)
+        if rt is None:
+            # getter
+            return mrcal.invert_rt(self._rt_cam_ref)
 
 
-    def extrinsics_rt_fromref(self, rt=None):
+        # setter
+        self._rt_cam_ref = mrcal.invert_rt(rt.astype(float))
+        return True
+
+
+    def rt_cam_ref(self, rt=None):
         r'''Get or set the extrinsics in this model
 
 SYNOPSIS
 
     # getter
-    rt_cr = model.extrinsics_rt_fromref()
+    rt_cr = model.rt_cam_ref()
 
     # setter
-    model.extrinsics_rt_fromref( rt_cr )
+    model.rt_cam_ref( rt_cr )
 
 This function gets/sets rt_fromref: a numpy array of shape (6,) describing the
 rt transformation FROM the reference coordinate system TO the camera coordinate
@@ -1526,62 +1642,26 @@ If this is a getter (no arguments given), returns a a numpy array of shape (6,).
 The rt transformation FROM the reference coordinate system.
 
         '''
-        return self._extrinsics_rt(False, rt)
-
-
-    def _extrinsics_Rt(self, toref, Rt=None):
-        r'''Get or set the extrinsics in this model
-
-        This function represents the pose as a shape (4,3) numpy array that
-        contains a (3,3) rotation matrix, followed by a (1,3) translation in the
-        last row:
-
-          R = Rt[:3,:]
-          t = Rt[ 3,:]
-
-        The transformation is b <-- R*a + t:
-
-          import numpysane as nps
-          b = nps.matmult(a, nps.transpose(R)) + t
-
-        if Rt is None: this is a getter; otherwise a setter.
-
-        toref is a boolean. if toref: then Rt maps points in the coord system of
-        THIS camera to the reference coord system. Otherwise in the opposite
-        direction
-
-        '''
-
-        # The internal representation is rt_fromref
-
-        if Rt is None:
+        if rt is None:
             # getter
-            rt_fromref = self._extrinsics
-            Rt_fromref = mrcal.Rt_from_rt(rt_fromref)
-            if not toref:
-                return Rt_fromref
-            return mrcal.invert_Rt(Rt_fromref)
+            return np.array(self._rt_cam_ref)
+
 
         # setter
-        if toref:
-            Rt_fromref = mrcal.invert_Rt(Rt.astype(float))
-            self._extrinsics = mrcal.rt_from_Rt(Rt_fromref)
-            return True
-
-        self._extrinsics = mrcal.rt_from_Rt(Rt.astype(float))
+        self._rt_cam_ref = np.array(rt, dtype=float)
         return True
 
 
-    def extrinsics_Rt_toref(self, Rt=None):
+    def Rt_ref_cam(self, Rt=None):
         r'''Get or set the extrinsics in this model
 
 SYNOPSIS
 
     # getter
-    Rt_rc = model.extrinsics_Rt_toref()
+    Rt_rc = model.Rt_ref_cam()
 
     # setter
-    model.extrinsics_Rt_toref( Rt_rc )
+    model.Rt_ref_cam( Rt_rc )
 
 This function gets/sets Rt_toref: a numpy array of shape (4,3) describing the Rt
 transformation FROM the camera coordinate system TO the reference coordinate
@@ -1603,19 +1683,25 @@ If this is a getter (no arguments given), returns a a numpy array of shape
 (4,3). The Rt transformation TO the reference coordinate system.
 
         '''
-        return self._extrinsics_Rt(True, Rt)
+        if Rt is None:
+            # getter
+            return mrcal.invert_Rt(mrcal.Rt_from_rt(self._rt_cam_ref))
+
+        # setter
+        self._rt_cam_ref = mrcal.rt_from_Rt(mrcal.invert_Rt(Rt.astype(float)))
+        return True
 
 
-    def extrinsics_Rt_fromref(self, Rt=None):
+    def Rt_cam_ref(self, Rt=None):
         r'''Get or set the extrinsics in this model
 
 SYNOPSIS
 
     # getter
-    Rt_cr = model.extrinsics_Rt_fromref()
+    Rt_cr = model.Rt_cam_ref()
 
     # setter
-    model.extrinsics_Rt_fromref( Rt_cr )
+    model.Rt_cam_ref( Rt_cr )
 
 This function gets/sets Rt_fromref: a numpy array of shape (4,3) describing the
 Rt transformation FROM the reference coordinate system TO the camera coordinate
@@ -1637,8 +1723,30 @@ If this is a getter (no arguments given), returns a a numpy array of shape
 (4,3). The Rt transformation FROM the reference coordinate system.
 
         '''
-        return self._extrinsics_Rt(False, Rt)
+        if Rt is None:
+            # getter
+            return mrcal.Rt_from_rt(self._rt_cam_ref)
 
+        # setter
+        self._rt_cam_ref = mrcal.rt_from_Rt(Rt.astype(float))
+        return True
+
+
+    def extrinsics_rt_toref(self, *args, **kwargs):
+        r'''Legacy alias for rt_ref_cam(); see the docs for that function'''
+        return self.rt_ref_cam(*args, **kwargs)
+
+    def extrinsics_rt_fromref(self, *args, **kwargs):
+        r'''Legacy alias for rt_cam_ref(); see the docs for that function'''
+        return self.rt_cam_ref(*args, **kwargs)
+
+    def extrinsics_Rt_toref(self, *args, **kwargs):
+        r'''Legacy alias for Rt_ref_cam(); see the docs for that function'''
+        return self.Rt_ref_cam(*args, **kwargs)
+
+    def extrinsics_Rt_fromref(self, *args, **kwargs):
+        r'''Legacy alias for Rt_cam_ref(); see the docs for that function'''
+        return self.Rt_cam_ref(*args, **kwargs)
 
     def imagersize(self, *args, **kwargs):
         r'''Get the imagersize in this model
@@ -1705,11 +1813,7 @@ If this is a getter (no arguments given), returns a numpy array of shape
             return np.array(self._valid_intrinsics_region, dtype=float)
 
         # setter
-        if valid_intrinsics_region is None:
-            self._valid_intrinsics_region = None
-            return True
-
-        valid_intrinsics_region = mrcal.close_contour(valid_intrinsics_region)
+        valid_intrinsics_region = mrcal.close_contour(np.array(valid_intrinsics_region))
 
         # raises exception on error
         _validateValidIntrinsicsRegion(valid_intrinsics_region)
@@ -1717,6 +1821,33 @@ If this is a getter (no arguments given), returns a numpy array of shape
         return True
 
 
+    def valid_intrinsics_region_reset(self):
+        r'''Reset the valid-intrinsics region to "none defined"
+
+SYNOPSIS
+
+    model.valid_intrinsics_region_reset()
+
+    print(model.valid_intrinsics_region() is None)
+    ----> True
+
+Clears out the valid-intrinsics region stored in the model. This is needed
+because cameramodel.valid_intrinsics_region(None) will return the current
+valid-intrinsics region instead of setting it to None
+
+ARGUMENTS
+
+None
+
+RETURNED VALUE
+
+None
+
+        '''
+        self._valid_intrinsics_region = None
+
+
+
     def optimization_inputs(self):
         r'''Get the original optimization inputs
 
@@ -1747,11 +1878,33 @@ The optimization_inputs dict, or None if one isn't stored in this model.
         if self._optimization_inputs_string is None:
             return None
         x = _deserialize_optimization_inputs(self._optimization_inputs_string)
-        if x['extrinsics_rt_fromref'] is None:
-            x['extrinsics_rt_fromref'] = np.zeros((0,6), dtype=float)
+        x['verbose'] = False
         return x
 
 
+    def optimization_inputs_reset(self):
+        r'''Reset the original optimization inputs to "none defined"
+
+SYNOPSIS
+
+    model.optimization_inputs_reset()
+
+    print(model.optimization_inputs() is None)
+    ----> True
+
+Clears out the optimization inputs stored in the model
+
+ARGUMENTS
+
+None
+
+RETURNED VALUE
+
+None
+        '''
+        self._optimization_inputs_string = None
+
+
     def _optimization_inputs_match(self, other_model):
         if self.       _optimization_inputs_string is None:
             if other_model._optimization_inputs_string is None:
@@ -1774,7 +1927,7 @@ The optimization_inputs dict, or None if one isn't stored in this model.
             mrcal.corresponding_icam_extrinsics(self.icam_intrinsics(),
                                                 **optimization_inputs)
 
-        rt_cam_ref = self.extrinsics_rt_fromref()
+        rt_cam_ref = self.rt_cam_ref()
 
         if icam_extrinsics < 0:
             # extrinsics WERE at the reference. So I should have an identity
@@ -1782,7 +1935,7 @@ The optimization_inputs dict, or None if one isn't stored in this model.
             return np.max(np.abs(rt_cam_ref)) > 0.0
 
         d = rt_cam_ref - \
-            optimization_inputs['extrinsics_rt_fromref'][icam_extrinsics]
+            optimization_inputs['rt_cam_ref'][icam_extrinsics]
         return np.max(np.abs(d)) > 1e-6
 
 
@@ -1802,8 +1955,11 @@ SYNOPSIS
                                             **optimization_inputs)
 
     if icam_extrinsics >= 0:
-        extrinsics_rt_fromref_at_calibration_time = \
-            optimization_inputs['extrinsics_rt_fromref'][icam_extrinsics]
+        rt_cam_ref_at_calibration_time = \
+            optimization_inputs['rt_cam_ref'][icam_extrinsics]
+    else:
+        rt_cam_ref_at_calibration_time = \
+            mrcal.identity_rt()
 
 This function retrieves the integer identifying this camera in the solve defined
 by 'optimization_inputs'. When the optimization happened, we may have been
