@@ -50,6 +50,23 @@ gradients, pass get_gradients=True. Logic:
 
 This function supports broadcasting fully.
 
+Note that the rotation r is not unique: the rotation angle (the magnitude of r)
+is only unique modulo 2 pi. And even then, rotations around axis v with angle
+pi+th are equivalent to rotations around axis -v with angle pi-th. So usually we
+will return r with mag(r) in [0,pi]. If the rotation angle is near pi, the
+returned r might have a magnitude slightly larger than pi.
+
+Furthermore, the gradient dr/dR needs a note. R is a (3,3) matrix, but not all
+(3,3) matrices are valid rotations. Let's define vecR as a flattened (9,) vector
+of all the values in R. The local linearization
+
+  delta_r = dr/dvecR delta_vecR
+
+is only meaningful if delta_vecR is in the valid-rotation subspace. Thus the
+gradient returned by this function is any dr/dvecR + K where K maps
+out-of-valid-R directions to 0. The matrix returned by this function CANNOT be
+assumed to have K = 0.
+
 ARGUMENTS
 
 - R: array of shape (3,3). This matrix defines the rotation. It is assumed that
@@ -236,6 +253,9 @@ Note that the translation gradient isn't returned: it is always the identity
 
 This function supports broadcasting fully.
 
+Some details about the returned r and gradients of dR are described in the
+documentation for r_from_R()
+
 ARGUMENTS
 
 - Rt: array of shape (4,3). This matrix defines the transformation. Rt[:3,:] is
@@ -531,7 +551,7 @@ and the gradients (rt, drtout/drtin)
         return _poseutils_npsp._invert_rt_withgrad(rt, out=out)
     return _poseutils_npsp._invert_rt(rt, out=out)
 
-def compose_Rt(*Rt, out=None):
+def compose_Rt(*Rt, out=None, inverted0=False, inverted1=False):
     r"""Compose Rt transformations
 
 SYNOPSIS
@@ -581,6 +601,10 @@ ARGUMENTS
   transformations, but any number could be given here. Each broadcasted slice
   has shape (4,3).
 
+- inverted0,inverted1: optional booleans, defaulting to False. If True, the
+  opposite transform is used for Rt0 and/or Rt1 respectively. inverted=True is
+  only supported when exactly two transforms are given
+
 - out: optional argument specifying the destination. By default, a new numpy
   array is created and returned. To write the results into an existing (and
   possibly non-contiguous) array, specify it with the 'out' kwarg. If 'out' is
@@ -592,10 +616,16 @@ RETURNED VALUE
 An array of composed Rt transformations. Each broadcasted slice has shape (4,3)
 
     """
+    if len(Rt) == 2:
+        return _poseutils_npsp._compose_Rt(*Rt, out=out, inverted0=inverted0, inverted1=inverted1)
+
+    if inverted0 or inverted1:
+        raise Exception("compose_Rt(..., inverted...=True) is supported only if exactly 2 inputs are given")
+
     Rt1onwards = reduce( _poseutils_npsp._compose_Rt, Rt[1:] )
     return _poseutils_npsp._compose_Rt(Rt[0], Rt1onwards, out=out)
 
-def compose_r(*r, get_gradients=False, out=None):
+def compose_r(*r, get_gradients=False, out=None, inverted0=False, inverted1=False):
     r"""Compose angle-axis rotations
 
 SYNOPSIS
@@ -640,11 +670,22 @@ rotations at the same time.
 In-place operation is supported; the output array may be the same as either of
 the input arrays to overwrite the input.
 
+Note that the rotation r is not unique: the rotation angle (the magnitude of r)
+is only unique modulo 2 pi. And even then, rotations around axis v with angle
+pi+th are equivalent to rotations around axis -v with angle pi-th. So usually we
+will have r with mag(r) in [0,pi]. This function might return a rotation outside
+this range.
+
 ARGUMENTS
 
 - *r: a list of rotations to compose. Usually we'll be composing two rotations,
   but any number could be given here. Each broadcasted slice has shape (3,)
 
+- inverted0,inverted1: optional booleans, defaulting to False. If True, the
+  opposite rotation is used for r0 and/or r1 respectively. The gradients d(r0
+  r1)/dr0 and d(r0 r1)/dr1 are returned in respect to the input r0 and r1.
+  inverted=True is only supported when exactly two rotations are given
+
 - get_gradients: optional boolean. By default (get_gradients=False) we return an
   array of composed rotations. Otherwise we return a tuple of arrays of composed
   rotations and their gradients. Gradient reporting is only supported when
@@ -678,15 +719,19 @@ and the gradients (r=compose(r0,r1), dr/dr0, dr/dr1):
 
     """
 
-    if get_gradients:
-        if len(r) != 2:
+    if len(r) != 2:
+        if get_gradients:
             raise Exception("compose_r(..., get_gradients=True) is supported only if exactly 2 inputs are given")
-        return _poseutils_npsp._compose_r_withgrad(*r, out=out)
+        if inverted0 or inverted1:
+            raise Exception("compose_r(..., inverted...=True) is supported only if exactly 2 inputs are given")
+
+    if get_gradients:
+        return _poseutils_npsp._compose_r_withgrad(*r, out=out, inverted0=inverted0, inverted1=inverted1)
 
     r1onwards = reduce( _poseutils_npsp._compose_r, r[1:] )
-    return _poseutils_npsp._compose_r(r[0], r1onwards, out=out)
+    return _poseutils_npsp._compose_r(r[0], r1onwards, out=out, inverted0=inverted0, inverted1=inverted1)
 
-def compose_rt(*rt, get_gradients=False, out=None):
+def compose_rt(*rt, get_gradients=False, out=None, inverted0=False, inverted1=False):
     r"""Compose rt transformations
 
 SYNOPSIS
@@ -746,13 +791,13 @@ Note that the poseutils C API returns only
 - dr_dr1
 - dt_dr0
 - dt_dt1
+- dt/dr1
+- dt/dt0
 
 because
 
 - dr/dt0 is always 0
 - dr/dt1 is always 0
-- dt/dr1 is always 0
-- dt/dt0 is always the identity matrix
 
 This Python function, however fills in those constants to return the full (and
 more convenient) arrays.
@@ -763,12 +808,20 @@ transformations at the same time.
 In-place operation is supported; the output array may be the same as either of
 the input arrays to overwrite the input.
 
+Some details about the returned r are described in the documentation for
+compose_r()
+
 ARGUMENTS
 
 - *rt: a list of transformations to compose. Usually we'll be composing two
   transformations, but any number could be given here. Each broadcasted slice
   has shape (6,)
 
+- inverted0,inverted1: optional booleans, defaulting to False. If True, the
+  opposite transform is used for rt0 and/or rt1 respectively. The gradients
+  d(rt0 rt1)/drt0 and d(rt0 rt1)/drt1 are returned in respect to the input rt0
+  and rt1. inverted=True is only supported when exactly two transforms are given
+
 - get_gradients: optional boolean. By default (get_gradients=False) we return an
   array of composed transformations. Otherwise we return a tuple of arrays of
   composed transformations and their gradients. Gradient reporting is only
@@ -793,7 +846,7 @@ drt/drt0,drt/drt1):
 
 1. The composed transformation. Each broadcasted slice has shape (6,)
 
-2. The gradient drt/dr0. Each broadcasted slice has shape (6,6). The first
+2. The gradient drt/drt0. Each broadcasted slice has shape (6,6). The first
    dimension selects the element of rt, and the last dimension selects the
    element of rt0
 
@@ -803,13 +856,256 @@ drt/drt0,drt/drt1):
 
     """
 
-    if get_gradients:
-        if len(rt) != 2:
+    if len(rt) != 2:
+        if get_gradients:
             raise Exception("compose_rt(..., get_gradients=True) is supported only if exactly 2 inputs are given")
-        return _poseutils_npsp._compose_rt_withgrad(*rt, out=out)
+        if inverted0 or inverted1:
+            raise Exception("compose_rt(..., inverted...=True) is supported only if exactly 2 inputs are given")
+
+    if get_gradients:
+        return _poseutils_npsp._compose_rt_withgrad(*rt, out=out, inverted0=inverted0, inverted1=inverted1)
 
     rt1onwards = reduce( _poseutils_npsp._compose_rt, rt[1:] )
-    return _poseutils_npsp._compose_rt(rt[0], rt1onwards, out=out)
+    return _poseutils_npsp._compose_rt(rt[0], rt1onwards, out=out, inverted0=inverted0, inverted1=inverted1)
+
+def compose_rt_tinyrt0_gradientrt0(rt1, out=None):
+    r"""Special-case composition for the uncertainty computation
+
+SYNOPSIS
+
+    r1  = rotation_axis1 * rotation_magnitude1
+    rt1 = nps.glue(r1, t1,  axis=-1)
+
+    drt01_drt0 = compose_rt_tinyrt0_gradientrt0(rt1)
+
+    ### Another way to get the same thing (but possibly less efficiently)
+     _,drt01_drt0,_ = compose_rt(np.zeros((6,),),
+                                 rt1,
+                                 get_gradients=True)
+
+This is a special-case subset of compose_rt(). It is the same, except:
+
+- rt0 is assumed to be 0, so we don't ingest it, and we don't report the
+  composition result
+- we ONLY report the drt01/drt0 gradient
+
+This special-case function is a part of the projection uncertainty computation,
+so it exists separate from compose_rt(). See the documentation for compose_rt()
+for all the details.
+
+This function supports broadcasting fully
+
+ARGUMENTS
+
+- rt1: the second of the two transformations being composed. The first is an
+  identity, so it's not given as an argument
+
+- out: optional argument specifying the destination. By default, a new numpy
+  array is created and returned. To write the results into an existing (and
+  possibly non-contiguous) array, specify it with the 'out' kwarg
+
+RETURNED VALUE
+
+We return a single array of shape (..., 6,6): drt01/drt0
+
+    """
+
+    # R0 (R1 p + t1) + t0 = R0 R1 p + (R0 t1 + t0)
+    # -> R01 = R0 R1
+    # -> t01 = R0 t1 + t0
+
+    # At rt0 ~ identity we have:
+
+    #   dt01/dr0 = d(R0 t1)/dr0
+
+    # rotate_point_r_core() says that
+
+    #   const val_withgrad_t<N> cross[3] =
+    #       {
+    #           (rg[1]*x_ing[2] - rg[2]*x_ing[1])*sign,
+    #           (rg[2]*x_ing[0] - rg[0]*x_ing[2])*sign,
+    #           (rg[0]*x_ing[1] - rg[1]*x_ing[0])*sign
+    #       };
+    #   const val_withgrad_t<N> inner =
+    #       rg[0]*x_ing[0] +
+    #       rg[1]*x_ing[1] +
+    #       rg[2]*x_ing[2];
+
+    #   // Small rotation. I don't want to divide by 0, so I take the limit
+    #   //   lim(th->0, xrot) =
+    #   //     = x + cross(r, x) + r rt x lim(th->0, (1 - cos(th)) / (th*th))
+    #   //     = x + cross(r, x) + r rt x lim(th->0, sin(th) / (2*th))
+    #   //     = x + cross(r, x) + r rt x/2
+    #   for(int i=0; i<3; i++)
+    #       x_outg[i] =
+    #           x_ing[i] +
+    #           cross[i] +
+    #           rg[i]*inner / 2.;
+
+    # So t01 = t0 + t1 + linear(r0) + quadratic(r0)
+
+    # r0 ~ 0 so I ignore the quadratic term:
+
+    #   dt01/dr0 = d(cross(r0,t1))/dr0
+    #            = -d(cross(t1,r0))/dr0
+    #            = -d(skew_symmetric(t1) r0))/dr0
+    #            = -skew_symmetric(t1)
+
+    # Thus
+
+    #   drt01/drt0 = [ dr01/dr0  dr01/dt0  ] = [ dr01/dr0              0 ]
+    #                [ dt01/dr0  dt01/dt0  ] = [ -skew_symmetric(t1)   I ]
+
+    # I call a function to get dr01_dr0
+
+    out_shape = rt1.shape + (6,)
+    if out is None:
+        out = np.zeros(out_shape, dtype=float)
+    else:
+        if not out.shape == out_shape:
+            raise Exception(f"The given 'out' array has shape {out.shape} but rt1 has shape {rt1.shape}, so 'out' should have shape {out_shape}")
+        out[:] = 0
+
+    _poseutils_npsp.skew_symmetric(rt1[..., 3:],
+                                   out = out[..., 3:, :3])
+    out[..., 3:, :3] *= -1
+
+    out[..., 0+3, 0+3] = 1.
+    out[..., 1+3, 1+3] = 1.
+    out[..., 2+3, 2+3] = 1.
+
+    _poseutils_npsp.compose_r_tinyr0_gradientr0(rt1[..., :3],
+                                                out = out[..., :3, :3])
+
+    return out
+
+def compose_rt_tinyrt1_gradientrt1(rt0, out=None):
+    r"""Special-case composition for the uncertainty computation
+
+SYNOPSIS
+
+    r0  = rotation_axis0 * rotation_magnitude0
+    rt0 = nps.glue(r0, t0,  axis=-1)
+
+    drt01_drt1 = compose_rt_tinyrt1_gradientrt1(rt0)
+
+    ### Another way to get the same thing (but possibly less efficiently)
+     _,_,drt01_drt1 = compose_rt(rt0,
+                                 np.zeros((6,),),
+                                 get_gradients=True)
+
+This is a special-case subset of compose_rt(). It is the same, except:
+
+- rt1 is assumed to be 0, so we don't ingest it, and we don't report the
+  composition result
+- we ONLY report the drt01/drt1 gradient
+
+This special-case function is a part of the projection uncertainty computation,
+so it exists separate from compose_rt(). See the documentation for compose_rt()
+for all the details.
+
+This function supports broadcasting fully
+
+ARGUMENTS
+
+- rt0: the first of the two transformations being composed. The second is an
+  identity, so it's not given as an argument
+
+- out: optional argument specifying the destination. By default, a new numpy
+  array is created and returned. To write the results into an existing (and
+  possibly non-contiguous) array, specify it with the 'out' kwarg
+
+RETURNED VALUE
+
+We return a single array of shape (..., 6,6): drt01/drt1
+
+    """
+
+    # R0 (R1 p + t1) + t0 = R0 R1 p + (R0 t1 + t0)
+    # -> R01 = R0 R1
+    # -> t01 = R0 t1 + t0
+
+    # rotate_point_r_core() says that
+
+    #   const val_withgrad_t<N> cross[3] =
+    #       {
+    #           (rg[1]*x_ing[2] - rg[2]*x_ing[1])*sign,
+    #           (rg[2]*x_ing[0] - rg[0]*x_ing[2])*sign,
+    #           (rg[0]*x_ing[1] - rg[1]*x_ing[0])*sign
+    #       };
+    #   const val_withgrad_t<N> inner =
+    #       rg[0]*x_ing[0] +
+    #       rg[1]*x_ing[1] +
+    #       rg[2]*x_ing[2];
+
+    #   if(th2.x < 1e-10)
+    #   {
+    #     // Small rotation. I don't want to divide by 0, so I take the limit
+    #     //   lim(th->0, xrot) =
+    #     //     = x + cross(r, x) + r rt x lim(th->0, (1 - cos(th)) / (th*th))
+    #     //     = x + cross(r, x) + r rt x lim(th->0, sin(th) / (2*th))
+    #     //     = x + cross(r, x) + r rt x/2
+    #     for(int i=0; i<3; i++)
+    #         x_outg[i] =
+    #             x_ing[i] +
+    #             cross[i] +
+    #             rg[i]*inner / 2.;
+    #   }
+    #   else
+    #   {
+    #     xrot = x cos(th) + cross(r, x)*sin(th)/th + r rt x (1 - cos(th)) / (th*th)
+    #   }
+
+    # So if r0 is small:
+    #   t01      = t0 + t1 + cross(r0,t1) + r0/2 inner(r0,t1)
+    #   dt01/dt1 = I + skew_symmetric(r0) + outer(r0,r0)/2
+    # else:
+    #   t01      = t0 + t1 cos(th) + cross(r0,t1)*sin(th)/th + outer(r0,r0) t1 (1 - cos(th)) / (th*th)
+    #   dt01/dt1 = I cos(th) + skew_symmetric(r0)*sin(th)/th + outer(r0,r0) (1 - cos(th)) / (th*th)
+
+    #   drt01/drt1 = [ dr01/dr1  dr01/dt1  ] = [ dr01/dr1   0        ]
+    #                [ dt01/dr1  dt01/dt1  ] = [ 0          dt01/dt1 ]
+
+    out_shape = rt0.shape + (6,)
+    if out is None:
+        out = np.zeros(out_shape, dtype=float)
+    else:
+        if not out.shape == out_shape:
+            raise Exception(f"The given 'out' array has shape {out.shape} but rt0 has shape {rt0.shape}, so 'out' should have shape {out_shape}")
+        out[:] = 0
+
+
+    # dummy arrays
+    rt01       = np.zeros(rt0.shape,        dtype=float)
+    drt01_drt0 = np.zeros(rt0.shape + (6,), dtype=float)
+
+    return \
+        compose_rt(rt0,
+                   np.zeros((6,),),
+                   get_gradients=True,
+                   out = (rt01,drt01_drt0,out))[2]
+
+
+
+    _poseutils_npsp.skew_symmetric(rt0[..., :3],
+                                   out = out[..., 3:, 3:])
+
+    th2 = nps.norm2(rt0[..., :3])
+    s = np.array(th2 * 0.)
+    mask_r0_tiny = th2 < 1e-10
+    if np.any(mask_r0_tiny):
+        s[ mask_r0_tiny] = 0.5
+
+    if np.any(~mask_r0_tiny):
+        s[~mask_r0_tiny] = (1 - np.cos(np.sqrt(th2[~mask_r0_tiny])) / th2[~mask_r0_tiny])
+
+    out[..., 3:, 3:] += np.eye(3) + nps.outer(rt0[..., :3],rt0[..., :3]) * s
+
+    _poseutils_npsp.compose_r_tinyr1_gradientr1(rt0[..., :3],
+                                                out = out[..., :3, :3])
+
+    return out
+
 
 def rotate_point_r(r, x, *, get_gradients=False, out=None, inverted=False):
     r"""Rotate point(s) using a Rodrigues vector
@@ -835,17 +1131,9 @@ SYNOPSIS
     ===>
     [(10,3), (10,3,3), (10,3,3)]
 
-Rotate point(s) by a rotation matrix. The Rodrigues vector is converted to a
-rotation matrix internally, and then this function is a matrix multiplication. x
-is stored as a row vector (that's how numpy stores 1-dimensional arrays), but
-the multiplication works as if x was a column vector (to match linear algebra
-conventions):
-
-    rotate_point_r(r,x) = transpose( matmult(R(r), transpose(x))) =
-                        = matmult(x, transpose(R(r)))
-
-By default this function returns the rotated points only. If we also want
-gradients, pass get_gradients=True. Logic:
+Rotate point(s) x by a Rodrigues rotation r. By default this function returns
+the rotated points only. If we also want gradients, pass get_gradients=True.
+Logic:
 
     if not get_gradients: return u=r(x)
     else:                 return (u=r(x),du/dr,du/dx)
@@ -1191,7 +1479,6 @@ gradients (u=Rt(x),du/dRt,du/dx):
     return _poseutils_npsp._transform_point_Rt_withgrad(Rt,x, out=out, inverted=inverted)
 
 
-from . import _poseutils_scipy
 quat_from_R = _poseutils_scipy.quat_from_R
 
 def qt_from_Rt(Rt, *, out=None):
