@@ -242,6 +242,8 @@ def synthesize_board_observations(models,
                                   rt_ref_boardcenter,
                                   rt_ref_boardcenter__noiseradius,
                                   Nframes,
+                                  max_oblique_angle_deg = None,
+                                  pcamera_nominal_ref   = np.array((0,0,0), dtype=float),
 
                                   which = 'all-cameras-must-see-full-board'):
     r'''Produce synthetic chessboard observations
@@ -264,11 +266,11 @@ SYNOPSIS
           calobject_warp  = None,
 
           # mean board pose and the radius of the added uniform noise
-          rt_ref_boardcenter              = rt_ref_boardcenter
+          rt_ref_boardcenter              = rt_ref_boardcenter,
           rt_ref_boardcenter__noiseradius = rt_ref_boardcenter__noiseradius,
 
           # How many frames we want
-          Nframes = 100
+          Nframes = 100,
 
           which = 'some-cameras-must-see-half-board')
 
@@ -356,6 +358,15 @@ ARGUMENTS
     that produce observations that are AT LEAST HALF visible by AT LEAST ONE
     camera.
 
+- max_oblique_angle_deg: optional value, defaulting to None. If non-None, we
+  only return observations where the board normal is within this angle of the
+  vector to the nominal camera (at pcamera_nominal_ref). This ensures that the
+  boards all "face" the camera to a certain degree
+
+- pcamera_nominal_ref: optional vector, defaulting to (0,0,0). Used in
+  conjunction with max_oblique_angle_deg to make sure the observation angle
+  isn't too oblique
+
 RETURNED VALUES
 
 We return a tuple:
@@ -386,11 +397,11 @@ We return a tuple:
                                             rt_ref_boardcenter              = nps.glue(r, np.array((0,0,3.)), axis=-1),
                                             rt_ref_boardcenter__noiseradius = np.array((0,0,0., 0,0,0)),
                                             Nframes                         = 1) [1]
-    mrcal.show_geometry( models_or_extrinsics_rt_fromref = np.zeros((1,1,6), dtype=float),
-                         frames_rt_toref                 = mrcal.rt_from_Rt(Rt_ref_boardref),
-                         object_width_n                  = 20,
-                         object_height_n                 = 5,
-                         object_spacing                  = 0.1,
+    mrcal.show_geometry( models_or_rt_cam_ref = np.zeros((1,1,6), dtype=float),
+                         rt_ref_frame         = mrcal.rt_from_Rt(Rt_ref_boardref),
+                         object_width_n       = 20,
+                         object_height_n      = 5,
+                         object_spacing       = 0.1,
                          _set = 'xyplane 0',
                          wait = 1 )
     '''
@@ -429,6 +440,12 @@ We return a tuple:
     Rt_boardref_origboardref = mrcal.identity_Rt()
     Rt_boardref_origboardref[3,:] = -board_center
 
+    if max_oblique_angle_deg is not None:
+        max_cos_oblique_angle = np.cos(max_oblique_angle_deg * np.pi/180.)
+    else:
+        max_cos_oblique_angle = None
+
+
     def get_observation_chunk():
         '''Make Nframes observations, and return them all, even the out-of-view ones'''
 
@@ -455,21 +472,33 @@ We return a tuple:
         q = \
           nps.mv( \
             nps.cat( \
-              *[ mrcal.project( mrcal.transform_point_Rt(models[i].extrinsics_Rt_fromref(), boards_ref),
+              *[ mrcal.project( mrcal.transform_point_Rt(models[i].Rt_cam_ref(), boards_ref),
                                 *models[i].intrinsics()) \
                  for i in range(Ncameras) ]),
                   0,1 )
 
         return q,Rt_ref_boardref
 
-    def cull_out_of_view(q,Rt_ref_boardref,
-                         which):
+    def cull(q,Rt_ref_boardref,
+             which):
 
         # q               has shape (Nframes,Ncameras,Nh,Nw,2)
         # Rt_ref_boardref has shape (Nframes,4,3)
 
-        # I pick only those frames where at least one cameras sees the whole
-        # board
+        ######## Throw out extreme oblique views
+        if max_cos_oblique_angle is not None:
+            nref_position = Rt_ref_boardref[...,3,:] - pcamera_nominal_ref
+            nref_position /= nps.dummy(nps.mag(nref_position), -1)
+            nref_orientation = Rt_ref_boardref[...,:3,2]
+            costh = np.abs(nps.inner(nref_position, nref_orientation))
+            i = costh > max_cos_oblique_angle
+
+            q               = q[i]
+            Rt_ref_boardref = Rt_ref_boardref[i]
+
+
+        ######## I pick only those frames where at least one cameras sees the
+        ######## whole board
 
         # shape (Nframes,Ncameras,Nh,Nw)
         mask_visible = (q[..., 0] >= 0) * (q[..., 1] >= 0)
@@ -512,8 +541,8 @@ We return a tuple:
         q_here, Rt_ref_boardref_here = get_observation_chunk()
 
         q_here, Rt_ref_boardref_here = \
-            cull_out_of_view(q_here, Rt_ref_boardref_here,
-                             which)
+            cull(q_here, Rt_ref_boardref_here,
+                 which)
 
         q = nps.glue(q, q_here, axis=-5)
         Rt_ref_boardref = nps.glue(Rt_ref_boardref, Rt_ref_boardref_here, axis=-3)
@@ -604,7 +633,7 @@ assumes that all the geometry and intrinsics are perfect, and sets the
 observations to projections of that perfect geometry. If requested, perfect
 gaussian noise is then added to the observations.
 
-THIS FUNCTION MODIES THE INPUT OPTIMIZATION_INPUTS
+THIS FUNCTION MODIFIES THE INPUT OPTIMIZATION_INPUTS
 
 ARGUMENTS
 
@@ -662,7 +691,7 @@ None
         # shape (Nobservations,4,3)
         Rt_cam_ref = \
             nps.glue( mrcal.identity_Rt(),
-                      mrcal.Rt_from_rt(optimization_inputs['extrinsics_rt_fromref']),
+                      mrcal.Rt_from_rt(optimization_inputs['rt_cam_ref']),
                       axis = -3 ) \
             [ indices_point_camintrinsics_camextrinsics[:,2]+1 ]
 
