@@ -685,6 +685,8 @@ for that function for details. This internal function differs from compose_Rt():
             args_input       = ('Rt0', 'Rt1'),
             prototype_input  = ((4,3,), (4,3,)),
             prototype_output = (4,3),
+            extra_args = (("int", "inverted0", "false", "p"),
+                          ("int", "inverted1", "false", "p"),),
 
             Ccode_slice_eval = \
                 {np.float64:
@@ -694,7 +696,8 @@ for that function for details. This internal function differs from compose_Rt():
                            (const double*)data_slice__Rt0,
                            strides_slice__Rt0[0], strides_slice__Rt0[1],
                            (const double*)data_slice__Rt1,
-                           strides_slice__Rt1[0], strides_slice__Rt1[1] );
+                           strides_slice__Rt1[0], strides_slice__Rt1[1],
+                           *inverted0, *inverted1);
     return true;
 '''},
 )
@@ -713,6 +716,8 @@ for that function for details. This internal function differs from compose_r():
             args_input       = ('r0', 'r1'),
             prototype_input  = ((3,), (3,)),
             prototype_output = (3,),
+            extra_args = (("int", "inverted0", "false", "p"),
+                          ("int", "inverted1", "false", "p"),),
 
             Ccode_slice_eval = \
                 {np.float64:
@@ -724,7 +729,8 @@ for that function for details. This internal function differs from compose_r():
                            (const double*)data_slice__r0,
                            strides_slice__r0[0],
                            (const double*)data_slice__r1,
-                           strides_slice__r1[0] );
+                           strides_slice__r1[0],
+                           *inverted0, *inverted1);
     return true;
 '''},
 )
@@ -744,6 +750,8 @@ for that function for details. This internal function differs from compose_r():
             args_input       = ('r0', 'r1'),
             prototype_input  = ((3,), (3,)),
             prototype_output = ((3,), (3,3),(3,3)),
+            extra_args = (("int", "inverted0", "false", "p"),
+                          ("int", "inverted1", "false", "p"),),
 
             Ccode_slice_eval = \
                 {np.float64:
@@ -762,7 +770,8 @@ for that function for details. This internal function differs from compose_r():
                            (const double*)data_slice__r0,
                            strides_slice__r0[0],
                            (const double*)data_slice__r1,
-                           strides_slice__r1[0] );
+                           strides_slice__r1[0],
+                           *inverted0, *inverted1);
 
     return true;
 '''},
@@ -789,8 +798,10 @@ This is a special-case subset of compose_r(). It is the same, except:
 - we ONLY report the dr01/dr0 gradient
 
 This special-case function is a part of the projection uncertainty computation,
-so it exists by itself. See the documentation for compose_r() for all the
-details.
+so it exists separate from compose_r(). See the documentation for compose_r()
+for all the details.
+
+This function supports broadcasting fully.
 
 ARGUMENTS
 
@@ -798,13 +809,12 @@ ARGUMENTS
   identity, so it's not given
 
 - out: optional argument specifying the destination. By default, a new numpy
-  array(s) is created and returned. To write the results into an existing (and
-  possibly non-contiguous) array, specify it with the 'out' kwarg. 'out' is the
-  one numpy array we will write into
+  array is created and returned. To write the results into an existing (and
+  possibly non-contiguous) array, specify it with the 'out' kwarg
 
 RETURNED VALUE
 
-We return a single array: dr01/dr0
+We return a single array of shape (...,3,3): dr01/dr0
 
 """,
 
@@ -826,6 +836,65 @@ We return a single array: dr01/dr0
 '''},
 )
 
+m.function( "compose_r_tinyr1_gradientr1",
+    r"""Special-case rotation composition for the uncertainty computation
+
+SYNOPSIS
+
+    r0 = rotation_axis0 * rotation_magnitude0
+
+    dr01_dr1 = compose_r_tinyr1_gradientr1(r0)
+
+    ### Another way to get the same thing (but possibly less efficiently)
+     _,_,dr01_dr1 = compose_r(r0,
+                              np.zeros((3,),),
+                              get_gradients=True)
+
+This is a special-case subset of compose_r(). It is the same, except:
+
+- r1 is assumed to be 0, so we don't ingest it, and we don't report the
+  composition result
+- we ONLY report the dr01/dr1 gradient
+
+This special-case function is a part of the projection uncertainty computation,
+so it exists separate from compose_r(). See the documentation for compose_r()
+for all the details.
+
+This function supports broadcasting fully.
+
+ARGUMENTS
+
+- r0: the first of the two rotations being composed. The second rotation is an
+  identity, so it's not given
+
+- out: optional argument specifying the destination. By default, a new numpy
+  array is created and returned. To write the results into an existing (and
+  possibly non-contiguous) array, specify it with the 'out' kwarg
+
+RETURNED VALUE
+
+We return a single array of shape (...,3,3): dr01/dr1
+
+""",
+
+            args_input       = ('r0',),
+            prototype_input  = ((3,),),
+            prototype_output = (3,3),
+
+            Ccode_slice_eval = \
+                {np.float64:
+                 r'''
+    mrcal_compose_r_tinyr1_gradientr1_full(
+        // dr/dr1
+        &item__output(0,0),
+        strides_slice__output[0], strides_slice__output[1],
+        (const double*)data_slice__r0,
+        strides_slice__r0[0] );
+
+    return true;
+'''},
+)
+
 m.function( "_compose_rt",
             """Compose two rt transformations
 
@@ -841,6 +910,8 @@ for that function for details. This internal function differs from compose_rt():
             args_input       = ('rt0', 'rt1'),
             prototype_input  = ((6,), (6,)),
             prototype_output = (6,),
+            extra_args = (("int", "inverted0", "false", "p"),
+                          ("int", "inverted1", "false", "p"),),
 
             Ccode_slice_eval = \
                 {np.float64:
@@ -851,10 +922,13 @@ for that function for details. This internal function differs from compose_rt():
                            NULL,0,0,
                            NULL,0,0,
                            NULL,0,0,
+                           NULL,0,0,
+                           NULL,0,0,
                            (const double*)data_slice__rt0,
                            strides_slice__rt0[0],
                            (const double*)data_slice__rt1,
-                           strides_slice__rt1[0] );
+                           strides_slice__rt1[0],
+                           *inverted0, *inverted1);
     return true;
 '''},
 )
@@ -874,8 +948,6 @@ Note that the C library returns limited gradients:
 
 - dr/dt0 is not returned: it is always 0
 - dr/dt1 is not returned: it is always 0
-- dt/dr1 is not returned: it is always 0
-- dt/dt0 is not returned: it is always the identity matrix
 
 THIS function combines these into the full drtout_drt0,drtout_drt1 arrays
 
@@ -884,6 +956,8 @@ THIS function combines these into the full drtout_drt0,drtout_drt1 arrays
             args_input       = ('rt0', 'rt1'),
             prototype_input  = ((6,), (6,)),
             prototype_output = ((6,), (6,6),(6,6)),
+            extra_args = (("int", "inverted0", "false", "p"),
+                          ("int", "inverted1", "false", "p"),),
 
             # output1 is drt/drt0 = [ dr/dr0 dr/dt0 ]
             #                       [ dt/dr0 dt/dt0 ]
@@ -915,6 +989,14 @@ THIS function combines these into the full drtout_drt0,drtout_drt1 arrays
                            &item__output1(3,0),
                            strides_slice__output1[0], strides_slice__output1[1],
 
+                           // dt/dr1
+                           &item__output2(3,0),
+                           strides_slice__output2[0], strides_slice__output2[1],
+
+                           // dt/dt0
+                           &item__output1(3,3),
+                           strides_slice__output1[0], strides_slice__output1[1],
+
                            // dt/dt1
                            &item__output2(3,3),
                            strides_slice__output2[0], strides_slice__output2[1],
@@ -922,20 +1004,15 @@ THIS function combines these into the full drtout_drt0,drtout_drt1 arrays
                            (const double*)data_slice__rt0,
                            strides_slice__rt0[0],
                            (const double*)data_slice__rt1,
-                           strides_slice__rt1[0] );
+                           strides_slice__rt1[0],
+                           *inverted0, *inverted1 );
     for(int i=0; i<3; i++)
         for(int j=0; j<3; j++)
         {
             item__output1(i,  j+3) = 0;
-            item__output1(i+3,j+3) = 0;
             item__output2(i,  j+3) = 0;
-            item__output2(i+3,j  ) = 0;
         }
 
-    item__output1(3,3) = 1.;
-    item__output1(4,4) = 1.;
-    item__output1(5,5) = 1.;
-
     return true;
 '''},
 )
@@ -1081,4 +1158,110 @@ We return the matrix A in a (3,3) numpy array
 '''}
 )
 
+for w in ('weights', 'noweights'):
+    for kind in ('R01', 'Rt01'):
+
+        if w == 'weights':
+            args_input      = ('v0','v1','weights')
+            prototype_input = (('N',3,), ('N',3,), ('N',))
+            weightarg = "(double*)data_slice__weights"
+        else:
+            args_input      = ('v0','v1')
+            prototype_input = (('N',3,), ('N',3,))
+            weightarg = "NULL"
+
+        if kind == 'R01':
+            what             = 'vectors'
+            prototype_output = (3,3)
+            Nelements_output = 9
+        else:
+            what             = 'points'
+            prototype_output = (4,3)
+            Nelements_output = 12
+
+        m.function( f"_align_procrustes_{what}_{kind}_{w}",
+            r"""Compute a rotation to align two sets of direction vectors or points
+
+        This is the written-in-C Python extension module. Most of the time you want to
+        use the mrcal.poseutils wrapper module instead of this module directly. Any
+        functions not prefixed with "_" are meant to be called directly, without the
+        wrapper.
+
+        All functions are exported into the mrcal module. So you can call these via
+        mrcal._poseutils.fff() or mrcal.fff(). The latter is preferred.
+
+            """,
+                    args_input       = args_input,
+                    prototype_input  = prototype_input,
+                    prototype_output = prototype_output,
+
+                    Ccode_validate = r'''
+                    return CHECK_CONTIGUOUS_AND_SETERROR_ALL();''',
+
+                    Ccode_slice_eval = \
+                        {np.float64:
+                         rf'''
+            bool result =
+            mrcal_align_procrustes_{what}_{kind}((double*)data_slice__output,
+                                                 dims_slice__v0[0],
+                                                 (double*)data_slice__v0,
+                                                 (double*)data_slice__v1,
+                                                 {weightarg});
+
+            if(!result && 0.0 == *(double*)data_slice__output)
+            {{
+
+                // Poorly-defined problem. I indicate this with an all-zero
+                // output, but I return true. This allows us to process
+                // lots of data via broadcasting, without breaking ALL
+                // the slices if one slice is broken
+                memset((double*)data_slice__output, 0, {Nelements_output}*sizeof(double));
+                return true;
+            }}
+            return result;
+        '''}
+        )
+
+m.function( f"R_aligned_to_vector",
+    r'''Compute a rotation to map a given vector to [0,0,1]
+
+SYNOPSIS
+
+    # I have a plane that passes through a point p, and has a normal n. I
+    # compute a transformation from the world to a coord system aligned to the
+    # plane, with p at the origin. R_plane_world p + t_plane_world = 0:
+
+    Rt_plane_world = np.zeros((4,3), dtype=float)
+    Rt_plane_world[:3,:] = mrcal.R_aligned_to_vector(n)
+    Rt_plane_world[ 3,:] = -mrcal.rotate_point_R(Rt_plane_world[:3,:],p)
+
+This rotation is not unique: adding any rotation around v still maps v to
+[0,0,1]. An arbitrary acceptable rotation is returned.
+
+ARGUMENTS
+
+- v: a numpy array of shape (3,). The vector that the computed rotation maps to
+  [0,0,1]. Does not need to be normalized. Must be non-0
+
+RETURNED VALUES
+
+The rotation in a (3,3) array
+
+    ''',
+            args_input       = ('v',),
+            prototype_input  = ( (3,), ),
+            prototype_output = (3,3),
+
+            Ccode_validate = r'''
+            return CHECK_CONTIGUOUS_AND_SETERROR_ALL();''',
+
+            Ccode_slice_eval = \
+                {np.float64:
+                 rf'''
+    mrcal_R_aligned_to_vector((double*)data_slice__output,
+                              (double*)data_slice__v);
+    return true;
+'''}
+)
+
 m.write()
