@@ -14,7 +14,7 @@
 
 #include "mrcal.h"
 #include "minimath/minimath.h"
-#include "util.h"
+#include "_util.h"
 
 // The equivalent function in Python is _rectified_resolution_python() in
 // stereo.py
@@ -174,7 +174,7 @@ bool mrcal_rectified_resolution( // output and input
 // The equivalent function in Python is _rectified_system_python() in stereo.py
 //
 // Documentation is in the docstring of mrcal.rectified_system()
-bool mrcal_rectified_system(// output
+bool mrcal_rectified_system2(// output
                             unsigned int*     imagersize_rectified,
                             double*           fxycxy_rectified,
                             double*           rt_rect0_ref,
@@ -192,6 +192,9 @@ bool mrcal_rectified_system(// output
                             mrcal_point2_t* azel0_deg,
 
                             // input
+                            // we refuse to create a view this close to az=90 or az=-90
+                            const double             az_edge_margin_deg,
+                            // Intrinsics used only for pixels_per_deg detection
                             const mrcal_lensmodel_t* lensmodel0,
                             const double*            intrinsics0,
 
@@ -207,17 +210,17 @@ bool mrcal_rectified_system(// output
 {
     if(el0_deg_autodetect)
     {
-        MSG("el0_deg_autodetect is unsupported");
+        MSG("el0_deg_autodetect is not yet supported");
         return false;
     }
     if(az_fov_deg_autodetect)
     {
-        MSG("az_fov_deg_autodetect is unsupported");
+        MSG("az_fov_deg_autodetect is not yet supported");
         return false;
     }
     if(el_fov_deg_autodetect)
     {
-        MSG("el_fov_deg_autodetect is unsupported");
+        MSG("el_fov_deg_autodetect is not yet supported");
         return false;
     }
 
@@ -229,7 +232,7 @@ bool mrcal_rectified_system(// output
         return false;
     }
 
-    mrcal_lensmodel_metadata_t meta =
+    const mrcal_lensmodel_metadata_t meta =
         mrcal_lensmodel_metadata( lensmodel0 );
     if(meta.noncentral)
     {
@@ -312,6 +315,13 @@ bool mrcal_rectified_system(// output
     *baseline = 0.0;
     for(int i=0; i<3; i++) *baseline += right[i]*right[i];
     *baseline = sqrt(*baseline);
+
+    if(*baseline < 1e-6)
+    {
+        MSG("The stereo pair has a unnaturally small baseline. Did you accidentally pass the same model for the two cameras?");
+        return false;
+    }
+
     for(int i=0; i<3; i++) right[i] /= (*baseline);
 
     // "forward" of the rectified coord system, in camera0 coords. The mean
@@ -339,6 +349,9 @@ bool mrcal_rectified_system(// output
     // Done with the geometry! Now to get the az/el grid. I need to figure
     // out the resolution and the extents
 
+    // I loosen the checks a bit, so that the checks pass after the correction
+    // despite any float error that might exist
+    const double az_edge_margin_deg_loose = az_edge_margin_deg - 1e-3;
     mrcal_point2_t azel0 = { .x = azel0_deg->x * M_PI/180.,
                              .y = azel0_deg->y * M_PI/180. };
     if(az0_deg_autodetect)
@@ -372,7 +385,44 @@ bool mrcal_rectified_system(// output
         for(int i=0; i<3; i++) norm2_forward01 += forward01[i]*forward01[i];
         azel0.x = asin( forward01_proj_right / sqrt(norm2_forward01) );
         azel0_deg->x = azel0.x * 180./M_PI;
+
+        if( !(azel0_deg->x - azel_fov_deg->x/2. > -90.+az_edge_margin_deg_loose &&
+              azel0_deg->x + azel_fov_deg->x/2. <  90.-az_edge_margin_deg_loose) )
+        {
+            // The detected az0 makes us look along the baseline. I shift it to
+            // avoid that
+            if(azel_fov_deg->x > 180. - 2.*az_edge_margin_deg)
+            {
+                MSG("ERROR: rectified view cannot avoid looking along the baseline vector: az_fov_deg is too large. Have az_fov=%.1fdeg angle_margin=%.1fdeg",
+                    azel_fov_deg->x, az_edge_margin_deg);
+                return false;
+            }
+
+            if(!(azel0_deg->x - azel_fov_deg->x/2. > -90.+az_edge_margin_deg_loose))
+            {
+                // We're off on this side
+                azel0_deg->x = -90.+az_edge_margin_deg + azel_fov_deg->x/2.;
+            }
+            else
+            {
+                // Off on the other side
+                azel0_deg->x = 90.-az_edge_margin_deg - azel_fov_deg->x/2.;
+            }
+
+            azel0.x = azel0_deg->x /180.*M_PI;
+        }
     }
+    else
+    {
+        if( !(azel0_deg->x - azel_fov_deg->x/2. > -90.+az_edge_margin_deg_loose &&
+              azel0_deg->x + azel_fov_deg->x/2. <  90.-az_edge_margin_deg_loose) )
+        {
+            MSG("ERROR: rectified view looks along the baseline vector. Reduce az_fov_deg or move az0_deg closer to the center. Have az0=%.1fdeg az_fov=%.1fdeg angle_margin=%.1fdeg",
+                azel0_deg->x, azel_fov_deg->x, az_edge_margin_deg);
+            return false;
+        }
+    }
+
 
     double R_cam0_rect0[3*3];
     mrcal_invert_R(R_cam0_rect0, R_rect0_cam0);
@@ -431,6 +481,69 @@ bool mrcal_rectified_system(// output
     return true;
 }
 
+// legacy function. Uses az_edge_margin_deg=10.0 without providing an argument
+bool mrcal_rectified_system(// output
+                            unsigned int*     imagersize_rectified,
+                            double*           fxycxy_rectified,
+                            double*           rt_rect0_ref,
+                            double*           baseline,
+
+                            // input, output
+                            // > 0: use given value
+                            // < 0: autodetect and scale
+                            double* pixels_per_deg_az,
+                            double* pixels_per_deg_el,
+
+                            // input, output
+                            // if(..._autodetect) { the results are returned here }
+                            mrcal_point2_t* azel_fov_deg,
+                            mrcal_point2_t* azel0_deg,
+
+                            // input
+                            // Intrinsics used only for pixels_per_deg detection
+                            const mrcal_lensmodel_t* lensmodel0,
+                            const double*            intrinsics0,
+
+                            const double*            rt_cam0_ref,
+                            const double*            rt_cam1_ref,
+
+                            const mrcal_lensmodel_type_t rectification_model_type,
+
+                            bool   az0_deg_autodetect,
+                            bool   el0_deg_autodetect,
+                            bool   az_fov_deg_autodetect,
+                            bool   el_fov_deg_autodetect)
+{
+    const double az_edge_margin_deg = 10.0;
+    return mrcal_rectified_system2(// output
+                            imagersize_rectified,
+                            fxycxy_rectified,
+                            rt_rect0_ref,
+                            baseline,
+                            // input, output
+                            // > 0: use given value
+                            // < 0: autodetect and scale
+                            pixels_per_deg_az,
+                            pixels_per_deg_el,
+                            // input, output
+                            // if(..._autodetect) { the results are returned here }
+                            azel_fov_deg,
+                            azel0_deg,
+
+                            // input
+                            az_edge_margin_deg,
+                            lensmodel0,
+                            intrinsics0,
+                            rt_cam0_ref,
+                            rt_cam1_ref,
+                            rectification_model_type,
+                            az0_deg_autodetect,
+                            el0_deg_autodetect,
+                            az_fov_deg_autodetect,
+                            el_fov_deg_autodetect);
+}
+
+
 static
 void set_rectification_map_pixel(// output
                                  float* rectification_map0,
@@ -490,9 +603,6 @@ bool mrcal_rectification_maps(// output
                               const unsigned int*          imagersize_rectified,
                               const double*                r_rect0_ref)
 {
-    ///// TODAY this C implementation supports MRCAL_LENSMODEL_LATLON only. This
-    ///// isn't a design choice, I just don't want to do the extra work yet. The
-    ///// API already is general enough to support both rectification schemes.
     if( ! (rectification_model_type == MRCAL_LENSMODEL_LATLON ||
            rectification_model_type == MRCAL_LENSMODEL_PINHOLE) )
     {
