@@ -20,22 +20,6 @@ import mrcal
 import testutils
 
 
-
-import tempfile
-import atexit
-import shutil
-workdir = tempfile.mkdtemp()
-def cleanup():
-    global workdir
-    try:
-        shutil.rmtree(workdir)
-        workdir = None
-    except:
-        pass
-atexit.register(cleanup)
-
-
-
 def check(filename_from, model_from,
           lensmodel_to,
           args, *,
@@ -45,7 +29,6 @@ def check(filename_from, model_from,
     process = \
         subprocess.Popen( (f"{testdir}/../mrcal-convert-lensmodel",
                            *args,
-                           "--outdir", workdir,
                            lensmodel_to),
                           encoding = 'ascii',
                           stdin    = subprocess.PIPE,
@@ -76,7 +59,8 @@ def check(filename_from, model_from,
         difflen, diff, q0, implied_Rt10 = \
             mrcal.projection_diff( (model_converted, model_from),
                                    use_uncertainties = False,
-                                   distance          = distance)
+                                   distance          = distance,
+                                   focus_radius      = 800)
         icenter = np.array(difflen.shape) // 2
         ithird  = np.array(difflen.shape) // 3
 
