@@ -201,7 +201,7 @@ if args.make_documentation_plots:
     print(f"Will write documentation plots to {args.make_documentation_plots_path}-xxxx.pdf and .png and .svg")
 
     if terminal['svg'] is None: terminal['svg'] = 'svg size 800,600       noenhanced solid dynamic    font ",14"'
-    if terminal['pdf'] is None: terminal['pdf'] = 'pdf size 8in,6in       noenhanced solid color      font ",12"'
+    if terminal['pdf'] is None: terminal['pdf'] = 'pdf size 8in,6in       noenhanced solid color      font ",16"'
     if terminal['png'] is None: terminal['png'] = 'pngcairo size 1024,768 transparent noenhanced crop font ",12"'
 else:
     args.make_documentation_plots_extratitle = None
@@ -238,16 +238,15 @@ calobject_warp_true     = np.array((0.002, -0.005))
 np.random.seed(0)
 
 
-extrinsics_rt_fromref_true = np.zeros((args.Ncameras,6), dtype=float)
-extrinsics_rt_fromref_true[:,:3] = np.random.randn(args.Ncameras,3) * 0.1
-extrinsics_rt_fromref_true[:, 3] = args.baseline * np.arange(args.Ncameras)
-extrinsics_rt_fromref_true[:,4:] = np.random.randn(args.Ncameras,2) * 0.1
+rt_cam_ref_true = np.zeros((args.Ncameras,6), dtype=float)
+rt_cam_ref_true[:,:3] = np.random.randn(args.Ncameras,3) * 0.1
+rt_cam_ref_true[:, 3] = args.baseline * np.arange(args.Ncameras)
+rt_cam_ref_true[:,4:] = np.random.randn(args.Ncameras,2) * 0.1
 
 # cam0 is at the identity. This makes my life easy: I can assume that the
 # optimization_inputs returned by calibration_baseline() use the same ref
-# coordinate system as these transformations. I explicitly state this by passing
-# calibration_baseline(allow_nonidentity_cam0_transform=False) later
-extrinsics_rt_fromref_true[0] *= 0
+# coordinate system as these transformations.
+rt_cam_ref_true[0] *= 0
 
 
 # shape (Npoints,3)
@@ -342,13 +341,9 @@ cache_id = f"{args.fixed}-{args.model}-{args.Nframes}-{args.Nsamples}-{args.Ncam
 cache_file = f"/tmp/test-triangulation-uncertainty--{cache_id}.pickle"
 
 if args.cache is None or args.cache == 'write':
-    optimization_inputs_baseline,                          \
-    models_true, models_baseline,                          \
-    indices_frame_camintrinsics_camextrinsics,             \
-    lensmodel, Nintrinsics, imagersizes,                   \
-    intrinsics_true, extrinsics_true_mounted, frames_true, \
-    observations_true,                                     \
-    args.Nframes =                                         \
+    optimization_inputs_baseline, \
+    models_true,                  \
+    frames_true =                 \
         calibration_baseline(args.model,
                              args.Ncameras,
                              args.Nframes,
@@ -356,36 +351,41 @@ if args.cache is None or args.cache == 'write':
                              object_width_n,
                              object_height_n,
                              object_spacing,
-                             extrinsics_rt_fromref_true,
+                             rt_cam_ref_true,
                              calobject_warp_true,
                              fixedframes,
                              testdir,
-                             cull_left_of_center = args.cull_left_of_center,
-                             allow_nonidentity_cam0_transform = False)
+                             cull_left_of_center = args.cull_left_of_center)
+
+    lensmodel   = optimization_inputs_baseline['lensmodel']
+    imagersizes = optimization_inputs_baseline['imagersizes']
+    Nintrinsics = mrcal.lensmodel_num_params(lensmodel)
+
 else:
     with open(cache_file,"rb") as f:
         (optimization_inputs_baseline,
          models_true,
-         models_baseline,
-         indices_frame_camintrinsics_camextrinsics,
          lensmodel,
          Nintrinsics,
          imagersizes,
-         intrinsics_true,
-         extrinsics_true_mounted,
          frames_true,
-         observations_true,
          intrinsics_sampled,
          extrinsics_sampled_mounted,
          frames_sampled,
          calobject_warp_sampled) = pickle.load(f)
 
-baseline_rt_ref_frame = optimization_inputs_baseline['frames_rt_toref']
+
+models_baseline = \
+    [ mrcal.cameramodel( optimization_inputs = optimization_inputs_baseline,
+                         icam_intrinsics     = i) \
+      for i in range(args.Ncameras) ]
+
+baseline_rt_ref_frame = optimization_inputs_baseline['rt_ref_frame']
 
 icam0,icam1 = args.cameras
 
-Rt01_true = mrcal.compose_Rt(mrcal.Rt_from_rt(extrinsics_rt_fromref_true[icam0]),
-                             mrcal.invert_Rt(mrcal.Rt_from_rt(extrinsics_rt_fromref_true[icam1])))
+Rt01_true = mrcal.compose_Rt(mrcal.Rt_from_rt(rt_cam_ref_true[icam0]),
+                             mrcal.invert_Rt(mrcal.Rt_from_rt(rt_cam_ref_true[icam1])))
 Rt10_true = mrcal.invert_Rt(Rt01_true)
 
 # shape (Npoints,Ncameras,3)
@@ -397,7 +397,7 @@ p_triangulated_true_local = nps.xchg( nps.cat( p_triangulated_true0,
 # shape (Npoints,Ncameras,2)
 q_true = nps.xchg( np.array([ mrcal.project(p_triangulated_true_local[:,i,:],
                                             lensmodel,
-                                            intrinsics_true[args.cameras[i]]) \
+                                            models_true[args.cameras[i]].intrinsics()[1]) \
                             for i in range(2)]),
                  0,1)
 
@@ -405,9 +405,9 @@ q_true = nps.xchg( np.array([ mrcal.project(p_triangulated_true_local[:,i,:],
 p_triangulated0 = \
     triangulate_nograd(models_true[icam0].intrinsics()[1],
                        models_true[icam1].intrinsics()[1],
-                       models_true[icam0].extrinsics_rt_fromref(),
-                       models_true[icam0].extrinsics_rt_fromref(),
-                       models_true[icam1].extrinsics_rt_fromref(),
+                       models_true[icam0].rt_cam_ref(),
+                       models_true[icam0].rt_cam_ref(),
+                       models_true[icam1].rt_cam_ref(),
                        frames_true, frames_true,
                        q_true,
                        lensmodel,
@@ -420,9 +420,9 @@ testutils.confirm_equal(p_triangulated0, p_triangulated_true0,
 p_triangulated0 = \
     triangulate_nograd(models_baseline[icam0].intrinsics()[1],
                        models_baseline[icam1].intrinsics()[1],
-                       models_baseline[icam0].extrinsics_rt_fromref(),
-                       models_baseline[icam0].extrinsics_rt_fromref(),
-                       models_baseline[icam1].extrinsics_rt_fromref(),
+                       models_baseline[icam0].rt_cam_ref(),
+                       models_baseline[icam0].rt_cam_ref(),
+                       models_baseline[icam1].rt_cam_ref(),
                        baseline_rt_ref_frame, baseline_rt_ref_frame,
                        q_true,
                        lensmodel,
@@ -446,8 +446,8 @@ istate_i0,               \
 istate_i1,               \
 icam_extrinsics0,        \
 icam_extrinsics1,        \
-istate_e1,               \
-istate_e0 =              \
+istate_e0,               \
+istate_e1 =              \
     mrcal.triangulation._triangulation_uncertainty_internal(slices,
                                                             optimization_inputs_baseline,
                                                             0,0,
@@ -455,9 +455,9 @@ istate_e0 =              \
 
 ########## Gradient check
 dp_triangulated_di0_empirical = grad(lambda i0: triangulate_nograd(i0, models_baseline[icam1].intrinsics()[1],
-                                                                   models_baseline[icam0].extrinsics_rt_fromref(),
-                                                                   models_baseline[icam0].extrinsics_rt_fromref(),
-                                                                   models_baseline[icam1].extrinsics_rt_fromref(),
+                                                                   models_baseline[icam0].rt_cam_ref(),
+                                                                   models_baseline[icam0].rt_cam_ref(),
+                                                                   models_baseline[icam1].rt_cam_ref(),
                                                                    baseline_rt_ref_frame, baseline_rt_ref_frame,
                                                                    q_true,
                                                                    lensmodel,
@@ -465,38 +465,38 @@ dp_triangulated_di0_empirical = grad(lambda i0: triangulate_nograd(i0, models_ba
                                      models_baseline[icam0].intrinsics()[1],
                                      step = 1e-5)
 dp_triangulated_di1_empirical = grad(lambda i1: triangulate_nograd(models_baseline[icam0].intrinsics()[1],i1,
-                                                                   models_baseline[icam0].extrinsics_rt_fromref(),
-                                                                   models_baseline[icam0].extrinsics_rt_fromref(),
-                                                                   models_baseline[icam1].extrinsics_rt_fromref(),
+                                                                   models_baseline[icam0].rt_cam_ref(),
+                                                                   models_baseline[icam0].rt_cam_ref(),
+                                                                   models_baseline[icam1].rt_cam_ref(),
                                                                    baseline_rt_ref_frame, baseline_rt_ref_frame,
                                                                    q_true,
                                                                    lensmodel,
                                                                    stabilize_coords=args.stabilize_coords),
                                      models_baseline[icam1].intrinsics()[1])
 dp_triangulated_de1_empirical = grad(lambda e1: triangulate_nograd(models_baseline[icam0].intrinsics()[1], models_baseline[icam1].intrinsics()[1],
-                                                                   models_baseline[icam0].extrinsics_rt_fromref(),
-                                                                   models_baseline[icam0].extrinsics_rt_fromref(),
+                                                                   models_baseline[icam0].rt_cam_ref(),
+                                                                   models_baseline[icam0].rt_cam_ref(),
                                                                    e1,
                                                                    baseline_rt_ref_frame, baseline_rt_ref_frame,
                                                                    q_true,
                                                                    lensmodel,
                                                                    stabilize_coords=args.stabilize_coords),
-                                     models_baseline[icam1].extrinsics_rt_fromref())
+                                     models_baseline[icam1].rt_cam_ref())
 
 dp_triangulated_de0_empirical = grad(lambda e0: triangulate_nograd(models_baseline[icam0].intrinsics()[1], models_baseline[icam1].intrinsics()[1],
                                                                    e0,
-                                                                   models_baseline[icam0].extrinsics_rt_fromref(),
-                                                                   models_baseline[icam1].extrinsics_rt_fromref(),
+                                                                   models_baseline[icam0].rt_cam_ref(),
+                                                                   models_baseline[icam1].rt_cam_ref(),
                                                                    baseline_rt_ref_frame, baseline_rt_ref_frame,
                                                                    q_true,
                                                                    lensmodel,
                                                                    stabilize_coords=args.stabilize_coords),
-                                     models_baseline[icam0].extrinsics_rt_fromref())
+                                     models_baseline[icam0].rt_cam_ref())
 
 dp_triangulated_drtrf_empirical = grad(lambda rtrf: triangulate_nograd(models_baseline[icam0].intrinsics()[1], models_baseline[icam1].intrinsics()[1],
-                                                                       models_baseline[icam0].extrinsics_rt_fromref(),
-                                                                       models_baseline[icam0].extrinsics_rt_fromref(),
-                                                                       models_baseline[icam1].extrinsics_rt_fromref(),
+                                                                       models_baseline[icam0].rt_cam_ref(),
+                                                                       models_baseline[icam0].rt_cam_ref(),
+                                                                       models_baseline[icam1].rt_cam_ref(),
                                                                        rtrf, baseline_rt_ref_frame,
                                                                        q_true,
                                                                        lensmodel,
@@ -505,9 +505,9 @@ dp_triangulated_drtrf_empirical = grad(lambda rtrf: triangulate_nograd(models_ba
                                        step = 1e-5)
 
 dp_triangulated_dq_empirical = grad(lambda q: triangulate_nograd(models_baseline[icam0].intrinsics()[1], models_baseline[icam1].intrinsics()[1],
-                                                                 models_baseline[icam0].extrinsics_rt_fromref(),
-                                                                 models_baseline[icam0].extrinsics_rt_fromref(),
-                                                                 models_baseline[icam1].extrinsics_rt_fromref(),
+                                                                 models_baseline[icam0].rt_cam_ref(),
+                                                                 models_baseline[icam0].rt_cam_ref(),
+                                                                 models_baseline[icam1].rt_cam_ref(),
                                                                  baseline_rt_ref_frame, baseline_rt_ref_frame,
                                                                  q,
                                                                  lensmodel,
@@ -520,6 +520,7 @@ testutils.confirm_equal(dp_triangulated_dbstate[...,istate_i0:istate_i0+Nintrins
                         relative = True,
                         worstcase = True,
                         eps = 0.1,
+                        reldiff_eps = 1e-5,
                         msg = "Gradient check: dp_triangulated_dbstate[intrinsics0]")
 testutils.confirm_equal(dp_triangulated_dbstate[...,istate_i1:istate_i1+Nintrinsics],
                         dp_triangulated_di1_empirical,
@@ -658,6 +659,9 @@ if not did_sample:
       frames_sampled,             \
       points_sampled,             \
       calobject_warp_sampled,     \
+      q_noise_board_sampled,      \
+      q_noise_point_sampled,      \
+      b_sampled,                  \
       optimization_inputs_sampled ) = \
           calibration_sample( args.Nsamples,
                               optimization_inputs_baseline,
@@ -669,14 +673,10 @@ if not did_sample:
             pickle.dump((optimization_inputs_baseline,
                          models_true,
                          models_baseline,
-                         indices_frame_camintrinsics_camextrinsics,
                          lensmodel,
                          Nintrinsics,
                          imagersizes,
-                         intrinsics_true,
-                         extrinsics_true_mounted,
                          frames_true,
-                         observations_true,
                          intrinsics_sampled,
                          extrinsics_sampled_mounted,
                          frames_sampled,
@@ -718,7 +718,7 @@ else:
 
 p_triangulated_sampled0 = triangulate_nograd(intrinsics_sampled[...,icam0,:], intrinsics_sampled[...,icam1,:],
                                              extrinsics_sampled_cam0,
-                                             models_baseline[icam0].extrinsics_rt_fromref(),
+                                             models_baseline[icam0].rt_cam_ref(),
                                              extrinsics_sampled_cam1,
                                              frames_sampled, baseline_rt_ref_frame,
                                              q_sampled,
@@ -796,7 +796,7 @@ for ipt in range(2):
                                         # it, but then I'd need to collect more
                                         # samples. Non-major axes have more
                                         # relative error
-                                        eps_eigenvalues      = (0.11, 0.2, 0.3),
+                                        eps_eigenvalues      = (0.15, 0.2, 0.3),
                                         eps_eigenvectors_deg = 6)
 
 
@@ -902,6 +902,9 @@ if args.make_documentation_plots is not None:
                      multiplot = f'layout 1,2',
                      unset = 'grid',
                      **processoptions )
+            if processoptions.get('hardcopy') and extension == 'pdf':
+                os.system(f"pdfcrop {processoptions['hardcopy']}")
+
         else:
             # Interactive plotting, so no multiplots. Interactive plots
             for p in subplots:
@@ -917,6 +920,8 @@ if args.make_documentation_plots is not None:
                       xlabel = 'Variable index (left point x,y,z; right point x,y,z)',
                       ylabel = 'Variable index (left point x,y,z; right point x,y,z)',
                       **processoptions)
+        if processoptions.get('hardcopy') and extension == 'pdf':
+            os.system(f"pdfcrop {processoptions['hardcopy']}")
 
 
         processoptions = copy.deepcopy(processoptions_base)
@@ -958,6 +963,8 @@ if args.make_documentation_plots is not None:
                 xlabel          = "Range to the left triangulated point (m)",
                 ylabel          = "Frequency",
                 **processoptions)
+        if processoptions.get('hardcopy') and extension == 'pdf':
+            os.system(f"pdfcrop {processoptions['hardcopy']}")
 
         processoptions = copy.deepcopy(processoptions_base)
         binwidth = np.sqrt(Var_distance) / 4.
@@ -984,6 +991,8 @@ if args.make_documentation_plots is not None:
                 xlabel          = "Distance between triangulated points (m)",
                 ylabel          = "Frequency",
                 **processoptions)
+        if processoptions.get('hardcopy') and extension == 'pdf':
+            os.system(f"pdfcrop {processoptions['hardcopy']}")
 
     if args.make_documentation_plots:
         for extension in ('pdf','svg','png','gp'):
